@@ -1,9 +1,9 @@
 /*
- * "$Id: ipp.c,v 1.62 2002/05/16 13:44:50 mike Exp $"
+ * "$Id: ipp.c,v 1.66 2002/12/17 18:56:33 swdev Exp $"
  *
  *   IPP backend for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -94,6 +94,10 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   ipp_attribute_t *copies_sup;	/* copies-supported attribute */
   ipp_attribute_t *charset_sup;	/* charset-supported attribute */
   ipp_attribute_t *format_sup;	/* document-format-supported attribute */
+  ipp_attribute_t *printer_state;
+  				/* printer-state attribute */
+  ipp_attribute_t *printer_accepting;
+  				/* printer-is-accepting-jobs attribute */
   const char	*charset;	/* Character set to use */
   cups_lang_t	*language;	/* Default language */
   int		copies;		/* Number of copies remaining */
@@ -111,6 +115,20 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 
   setbuf(stderr, NULL);
 
+ /*
+  * Ignore SIGPIPE signals...
+  */
+
+#ifdef HAVE_SIGSET
+  sigset(SIGPIPE, SIG_IGN);
+#elif defined(HAVE_SIGACTION)
+  memset(&action, 0, sizeof(action));
+  action.sa_handler = SIG_IGN;
+  sigaction(SIGPIPE, &action, NULL);
+#else
+  signal(SIGPIPE, SIG_IGN);
+#endif /* HAVE_SIGSET */
+
  /*
   * Check command-line...
   */
@@ -199,8 +217,32 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   {
     fprintf(stderr, "INFO: Connecting to %s on port %d...\n", hostname, port);
 
-    if ((http = httpConnect(hostname, port)) == NULL)
+    if ((http = httpConnectEncrypt(hostname, port, cupsEncryption())) == NULL)
     {
+      if (getenv("CLASS") != NULL)
+      {
+       /*
+        * If the CLASS environment variable is set, the job was submitted
+	* to a class and not to a specific queue.  In this case, we want
+	* to abort immediately so that the job can be requeued on the next
+	* available printer in the class.
+	*/
+
+        fprintf(stderr, "INFO: Unable to queue job on %s, queuing on next printer in class...\n",
+	        hostname);
+
+        if (argc == 6)
+	  unlink(filename);
+
+       /*
+        * Sleep 5 seconds to keep the job from requeuing too rapidly...
+	*/
+
+	sleep(5);
+
+        return (1);
+      }
+
       if (errno == ECONNREFUSED || errno == EHOSTDOWN ||
           errno == EHOSTUNREACH)
       {
@@ -331,6 +373,49 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   }
   while (ipp_status > IPP_OK_CONFLICT);
 
+ /*
+  * See if the printer is accepting jobs and is not stopped; if either
+  * condition is true and we are printing to a class, requeue the job...
+  */
+
+  if (getenv("CLASS") != NULL)
+  {
+    printer_state     = ippFindAttribute(supported, "printer-state",
+                                	 IPP_TAG_ENUM);
+    printer_accepting = ippFindAttribute(supported, "printer-is-accepting-jobs",
+                                	 IPP_TAG_BOOLEAN);
+
+    if (printer_state == NULL ||
+	printer_state->values[0].integer > IPP_PRINTER_PROCESSING ||
+	printer_accepting == NULL ||
+	!printer_accepting->values[0].boolean)
+    {
+     /*
+      * If the CLASS environment variable is set, the job was submitted
+      * to a class and not to a specific queue.  In this case, we want
+      * to abort immediately so that the job can be requeued on the next
+      * available printer in the class.
+      */
+
+      fprintf(stderr, "INFO: Unable to queue job on %s, queuing on next printer in class...\n",
+	      hostname);
+
+      ippDelete(supported);
+      httpClose(http);
+
+      if (argc == 6)
+	unlink(filename);
+
+     /*
+      * Sleep 5 seconds to keep the job from requeuing too rapidly...
+      */
+
+      sleep(5);
+
+      return (1);
+    }
+  }
+
  /*
   * Now that we are "connected" to the port, ignore SIGTERM so that we
   * can finish out any page data the driver sends (e.g. to eject the
@@ -817,5 +902,5 @@ report_printer_state(ipp_t *ipp)	/* I - IPP response */
 
 
 /*
- * End of "$Id: ipp.c,v 1.62 2002/05/16 13:44:50 mike Exp $".
+ * End of "$Id: ipp.c,v 1.66 2002/12/17 18:56:33 swdev Exp $".
  */
