@@ -1,9 +1,9 @@
 /*
- * "$Id: lpd.c,v 1.36 2002/05/16 13:44:50 mike Exp $"
+ * "$Id: lpd.c,v 1.47 2002/12/17 18:56:33 swdev Exp $"
  *
  *   Line Printer Daemon backend for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -28,6 +28,7 @@
  *   main()        - Send a file to the printer or server.
  *   lpd_command() - Send an LPR command sequence and wait for a reply.
  *   lpd_queue()   - Queue a file using the Line Printer Daemon protocol.
+ *   lpd_timeout() - Handle timeout alarms...
  *   lpd_write()   - Write a buffer of data to an LPD server.
  */
 
@@ -78,7 +79,9 @@ extern int	rresvport(int *port);
 static int	lpd_command(int lpd_fd, char *format, ...);
 static int	lpd_queue(char *hostname, char *printer, char *filename,
 		          int fromstdin, char *user, char *title, int copies,
-			  int banner, int format, int order);
+			  int banner, int format, int order, int reserve,
+			  int manual_copies);
+static void	lpd_timeout(int sig);
 static int	lpd_write(int lpd_fd, char *buffer, int length);
 
 
@@ -109,6 +112,13 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   int	banner;		/* Print banner page? */
   int	format;		/* Print format */
   int	order;		/* Order of control/data files */
+  int	reserve;	/* Reserve priviledged port? */
+  int	manual_copies,	/* Do manual copies? */
+	copies;		/* Number of copies */
+#if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
+  struct sigaction action;
+			/* Actions for POSIX signals */
+#endif /* HAVE_SIGACTION && !HAVE_SIGSET */
 
 
  /*
@@ -117,6 +127,20 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 
   setbuf(stderr, NULL);
 
+ /*
+  * Ignore SIGPIPE signals...
+  */
+
+#ifdef HAVE_SIGSET
+  sigset(SIGPIPE, SIG_IGN);
+#elif defined(HAVE_SIGACTION)
+  memset(&action, 0, sizeof(action));
+  action.sa_handler = SIG_IGN;
+  sigaction(SIGPIPE, &action, NULL);
+#else
+  signal(SIGPIPE, SIG_IGN);
+#endif /* HAVE_SIGSET */
+
  /*
   * Check command-line...
   */
@@ -180,9 +204,11 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
   * See if there are any options...
   */
 
-  banner = 0;
-  format = 'l';
-  order  = ORDER_CONTROL_DATA;
+  banner        = 0;
+  format        = 'l';
+  order         = ORDER_CONTROL_DATA;
+  reserve       = 0;
+  manual_copies = 1;
 
   if ((options = strchr(resource, '?')) != NULL)
   {
@@ -264,6 +290,28 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 	else
 	  fprintf(stderr, "ERROR: Unknown file order \"%s\"\n", value);
       }
+      else if (strcasecmp(name, "reserve") == 0)
+      {
+       /*
+        * Set port reservation mode...
+	*/
+
+        reserve = !value[0] ||
+	          strcasecmp(value, "on") == 0 ||
+	 	  strcasecmp(value, "yes") == 0 ||
+	 	  strcasecmp(value, "true") == 0;
+      }
+      else if (strcasecmp(name, "manual_copies") == 0)
+      {
+       /*
+        * Set port reservation mode...
+	*/
+
+        manual_copies = !value[0] ||
+	        	strcasecmp(value, "on") == 0 ||
+	 		strcasecmp(value, "yes") == 0 ||
+	 		strcasecmp(value, "true") == 0;
+      }
     }
   }
 
@@ -283,16 +331,28 @@ main(int  argc,		/* I - Number of command-line arguments (6 or 7) */
 
   if (argc > 6)
   {
+    if (manual_copies)
+    {
+      manual_copies = atoi(argv[4]);
+      copies        = 1;
+    }
+    else
+    {
+      manual_copies = 1;
+      copies        = atoi(argv[4]);
+    }
+
     status = lpd_queue(hostname, resource + 1, filename, 0,
-                       argv[2] /* user */, title, atoi(argv[4]) /* copies */,
-		       banner, format, order);
+                       argv[2] /* user */, title, copies,
+		       banner, format, order, reserve, manual_copies);
 
     if (!status)
       fprintf(stderr, "PAGE: 1 %d\n", atoi(argv[4]));
   }
   else
     status = lpd_queue(hostname, resource + 1, filename, 1,
-                       argv[2] /* user */, title, 1, banner, format, order);
+                       argv[2] /* user */, title, 1,
+		       banner, format, order, reserve, 1);
 
  /*
   * Remove the temporary file if necessary...
@@ -340,6 +400,8 @@ lpd_command(int  fd,		/* I - Socket connection to LPD host */
 
   fprintf(stderr, "DEBUG: Sending command string (%d bytes)...\n", bytes);
 
+  alarm(30);
+
   if (lpd_write(fd, buf, bytes) < bytes)
     return (-1);
 
@@ -349,8 +411,12 @@ lpd_command(int  fd,		/* I - Socket connection to LPD host */
 
   fprintf(stderr, "DEBUG: Reading command status...\n");
 
+  alarm(30);
+
   if (recv(fd, &status, 1, 0) < 1)
-    return (-1);
+    status = errno;
+
+  alarm(0);
 
   fprintf(stderr, "DEBUG: lpd_command returning %d\n", status);
 
@@ -372,7 +438,9 @@ lpd_queue(char *hostname,	/* I - Host to connect to */
 	  int  copies,		/* I - Number of copies */
 	  int  banner,		/* I - Print LPD banner? */
           int  format,		/* I - Format specifier */
-          int  order)		/* I - Order of data/control files */
+          int  order,		/* I - Order of data/control files */
+	  int  reserve,		/* I - Reserve ports? */
+	  int  manual_copies)	/* I - Do copies by hand... */
 {
   FILE			*fp;		/* Job file */
   char			localhost[255];	/* Local host name */
@@ -385,6 +453,7 @@ lpd_queue(char *hostname,	/* I - Host to connect to */
   char			status;		/* Status byte from command */
   struct sockaddr_in	addr;		/* Socket address */
   struct hostent	*hostaddr;	/* Host address */
+  int			copy;		/* Copies written */
   size_t		nbytes,		/* Number of bytes written */
 			tbytes;		/* Total bytes written */
   char			buffer[8192];	/* Output buffer */
@@ -394,251 +463,346 @@ lpd_queue(char *hostname,	/* I - Host to connect to */
 
 
  /*
-  * First try to reserve a port for this connection...
+  * Setup an alarm handler for timeouts...
   */
 
-  if ((hostaddr = httpGetHostByName(hostname)) == NULL)
-  {
-    fprintf(stderr, "ERROR: Unable to locate printer \'%s\' - %s",
-            hostname, strerror(errno));
-    return (1);
-  }
-
-  fprintf(stderr, "INFO: Attempting to connect to host %s for printer %s\n",
-          hostname, printer);
-
-  memset(&addr, 0, sizeof(addr));
-  memcpy(&(addr.sin_addr), hostaddr->h_addr, hostaddr->h_length);
-  addr.sin_family = hostaddr->h_addrtype;
-  addr.sin_port   = htons(515);	/* LPD/printer service */
-
-  for (port = 732;;)
-  {
-    if (getuid())
-    {
-     /*
-      * We're running as a normal user, so just create a regular socket...
-      */
-
-      if ((fd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
-      {
-        perror("ERROR: Unable to create socket");
-        return (1);
-      }
-    }
-    else
-    {
-     /*
-      * We're running as root, so comply with RFC 1179 and reserve a
-      * priviledged port between 721 and 732...
-      */
-
-      if ((fd = rresvport(&port)) < 0)
-      {
-	perror("ERROR: Unable to reserve port");
-	sleep(30);
-	continue;
-      }
-    }
-
-    if (connect(fd, (struct sockaddr *)&addr, sizeof(addr)) < 0)
-    {
-      error = errno;
-      close(fd);
-      fd = -1;
-
-      if (error == ECONNREFUSED || error == EHOSTDOWN ||
-          error == EHOSTUNREACH)
-      {
-	fprintf(stderr, "INFO: Network host \'%s\' is busy; will retry in 30 seconds...",
-                hostname);
-	sleep(30);
-      }
-      else if (error == EADDRINUSE)
-      {
-	port --;
-	if (port < 721)
-	  port = 732;
-      }
-      else
-      {
-	perror("ERROR: Unable to connect to printer");
-        sleep(30);
-      }
-    }
-    else
-      break;
-  }
-
-  fprintf(stderr, "INFO: Connected from port %d...\n", port);
-
- /*
-  * Now that we are "connected" to the port, ignore SIGTERM so that we
-  * can finish out any page data the driver sends (e.g. to eject the
-  * current page...  Only ignore SIGTERM if we are printing data from
-  * stdin (otherwise you can't cancel raw jobs...)
-  */
-
-  if (fromstdin)
-  {
 #ifdef HAVE_SIGSET /* Use System V signals over POSIX to avoid bugs */
-    sigset(SIGTERM, SIG_IGN);
+  sigset(SIGALRM, lpd_timeout);
 #elif defined(HAVE_SIGACTION)
-    memset(&action, 0, sizeof(action));
+  memset(&action, 0, sizeof(action));
 
-    sigemptyset(&action.sa_mask);
-    action.sa_handler = SIG_IGN;
-    sigaction(SIGTERM, &action, NULL);
+  sigemptyset(&action.sa_mask);
+  action.sa_handler = lpd_timeout;
+  sigaction(SIGALRM, &action, NULL);
 #else
-    signal(SIGTERM, SIG_IGN);
+  signal(SIGALRM, lpd_timeout);
 #endif /* HAVE_SIGSET */
-  }
 
  /*
-  * Next, open the print file and figure out its size...
+  * Loop forever trying to print the file...
   */
 
-  if (stat(filename, &filestats))
-  {
-    perror("ERROR: unable to stat print file");
-    return (1);
-  }
-
-  if ((fp = fopen(filename, "rb")) == NULL)
-  {
-    perror("ERROR: unable to open print file for reading");
-    return (1);
-  }
-
- /*
-  * Send a job header to the printer, specifying no banner page and
-  * literal output...
-  */
-
-  lpd_command(fd, "\002%s\n", printer);	/* Receive print job(s) */
-
-  gethostname(localhost, sizeof(localhost));
-  localhost[31] = '\0'; /* RFC 1179, Section 7.2 - host name < 32 chars */
-
-  snprintf(control, sizeof(control), "H%s\nP%s\nJ%s\n", localhost, user, title);
-  cptr = control + strlen(control);
-
-  if (banner)
-  {
-    snprintf(cptr, sizeof(control) - (cptr - control), "L%s\n", user);
-    cptr   += strlen(cptr);
-  }
-
-  while (copies > 0)
-  {
-    snprintf(cptr, sizeof(control) - (cptr - control), "%cdfA%03d%s\n", format,
-             getpid() % 1000, localhost);
-    cptr   += strlen(cptr);
-    copies --;
-  }
-
-  snprintf(cptr, sizeof(control) - (cptr - control),
-           "UdfA%03d%s\nNdfA%03d%s\n",
-           getpid() % 1000, localhost,
-           getpid() % 1000, localhost);
-
-  fprintf(stderr, "DEBUG: Control file is:\n%s", control);
-
-  if (order == ORDER_CONTROL_DATA)
-  {
-    lpd_command(fd, "\002%d cfA%03.3d%s\n", strlen(control), getpid() % 1000,
-        	localhost);
-
-    fprintf(stderr, "INFO: Sending control file (%lu bytes)\n", (unsigned long)strlen(control));
-
-    if (lpd_write(fd, control, strlen(control) + 1) < (strlen(control) + 1))
-    {
-      status = errno;
-      perror("ERROR: Unable to write control file");
-    }
-    else if (read(fd, &status, 1) < 1)
-      status = errno;
-
-    if (status != 0)
-      fprintf(stderr, "ERROR: Remote host did not accept control file (%d)\n",
-              status);
-    else
-      fputs("INFO: Control file sent successfully\n", stderr);
-  }
-  else
-    status = 0;
-
-  if (status == 0)
+  for (;;) /* FOREVER */
   {
    /*
-    * Send the print file...
+    * First try to reserve a port for this connection...
     */
 
-    lpd_command(fd, "\003%u dfA%03.3d%s\n", (unsigned)filestats.st_size,
-                getpid() % 1000, localhost);
-
-    fprintf(stderr, "INFO: Sending data file (%u bytes)\n",
-            (unsigned)filestats.st_size);
-
-    tbytes = 0;
-    while ((nbytes = fread(buffer, 1, sizeof(buffer), fp)) > 0)
+    if ((hostaddr = httpGetHostByName(hostname)) == NULL)
     {
-      fprintf(stderr, "INFO: Spooling LPR job, %u%% complete...\n",
-              (unsigned)(100.0f * tbytes / filestats.st_size));
+      fprintf(stderr, "ERROR: Unable to locate printer \'%s\' - %s\n",
+              hostname, strerror(errno));
+      return (1);
+    }
 
-      if (lpd_write(fd, buffer, nbytes) < nbytes)
+    fprintf(stderr, "INFO: Attempting to connect to host %s for printer %s\n",
+            hostname, printer);
+
+    memset(&addr, 0, sizeof(addr));
+    memcpy(&(addr.sin_addr), hostaddr->h_addr, hostaddr->h_length);
+    addr.sin_family = hostaddr->h_addrtype;
+    addr.sin_port   = htons(515);	/* LPD/printer service */
+
+    for (port = 732;;)
+    {
+      if (getuid() || !reserve)
       {
-        perror("ERROR: Unable to send print file to printer");
-        break;
+       /*
+	* Just create a regular socket...
+	*/
+
+	if ((fd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
+	{
+          perror("ERROR: Unable to create socket");
+          return (1);
+	}
+
+	port = 515;
       }
       else
-        tbytes += nbytes;
+      {
+       /*
+	* We're running as root and want to comply with RFC 1179.  Reserve a
+	* priviledged port between 721 and 732...
+	*/
+
+	if ((fd = rresvport(&port)) < 0)
+	{
+	  perror("ERROR: Unable to reserve port");
+	  sleep(30);
+	  continue;
+	}
+      }
+
+      if (connect(fd, (struct sockaddr *)&addr, sizeof(addr)) < 0)
+      {
+	error = errno;
+	close(fd);
+	fd = -1;
+
+	if (error == ECONNREFUSED || error == EHOSTDOWN ||
+            error == EHOSTUNREACH)
+	{
+	  fprintf(stderr, "WARNING: Network host \'%s\' is busy, down, or unreachable; will retry in 30 seconds...\n",
+                  hostname);
+	  sleep(30);
+	}
+	else if (error == EADDRINUSE)
+	{
+	  port --;
+	  if (port < 721)
+	    port = 732;
+	}
+	else
+	{
+	  perror("ERROR: Unable to connect to printer");
+          sleep(30);
+	}
+      }
+      else
+	break;
     }
 
-    if (tbytes < filestats.st_size)
-      status = errno;
-    else if (lpd_write(fd, "", 1) < 1)
-      status = errno;
-    else if (recv(fd, &status, 1, 0) < 1)
-      status = errno;
+    fprintf(stderr, "INFO: Connected from port %d...\n", port);
 
-    if (status != 0)
-      fprintf(stderr, "ERROR: Remote host did not accept data file (%d)\n",
-              status);
-    else
-      fputs("INFO: Data file sent successfully\n", stderr);
-  }
+   /*
+    * Now that we are "connected" to the port, ignore SIGTERM so that we
+    * can finish out any page data the driver sends (e.g. to eject the
+    * current page...  Only ignore SIGTERM if we are printing data from
+    * stdin (otherwise you can't cancel raw jobs...)
+    */
 
-  if (status == 0 && order == ORDER_DATA_CONTROL)
-  {
-    lpd_command(fd, "\002%d cfA%03.3d%s\n", strlen(control), getpid() % 1000,
-        	localhost);
-
-    fprintf(stderr, "INFO: Sending control file (%lu bytes)\n", (unsigned long)strlen(control));
-
-    if (lpd_write(fd, control, strlen(control) + 1) < (strlen(control) + 1))
+    if (fromstdin)
     {
-      status = errno;
-      perror("ERROR: Unable to write control file");
+#ifdef HAVE_SIGSET /* Use System V signals over POSIX to avoid bugs */
+      sigset(SIGTERM, SIG_IGN);
+#elif defined(HAVE_SIGACTION)
+      memset(&action, 0, sizeof(action));
+
+      sigemptyset(&action.sa_mask);
+      action.sa_handler = SIG_IGN;
+      sigaction(SIGTERM, &action, NULL);
+#else
+      signal(SIGTERM, SIG_IGN);
+#endif /* HAVE_SIGSET */
     }
-    else if (read(fd, &status, 1) < 1)
-      status = errno;
 
-    if (status != 0)
-      fprintf(stderr, "ERROR: Remote host did not accept control file (%d)\n",
-              status);
+   /*
+    * Next, open the print file and figure out its size...
+    */
+
+    if (stat(filename, &filestats))
+    {
+      perror("ERROR: unable to stat print file");
+      return (1);
+    }
+
+    filestats.st_size *= manual_copies;
+
+    if ((fp = fopen(filename, "rb")) == NULL)
+    {
+      perror("ERROR: unable to open print file for reading");
+      return (1);
+    }
+
+   /*
+    * Send a job header to the printer, specifying no banner page and
+    * literal output...
+    */
+
+    lpd_command(fd, "\002%s\n", printer);	/* Receive print job(s) */
+
+    gethostname(localhost, sizeof(localhost));
+    localhost[31] = '\0'; /* RFC 1179, Section 7.2 - host name < 32 chars */
+
+    snprintf(control, sizeof(control), "H%s\nP%s\nJ%s\n", localhost, user, title);
+    cptr = control + strlen(control);
+
+    if (banner)
+    {
+      snprintf(cptr, sizeof(control) - (cptr - control), "L%s\n", user);
+      cptr   += strlen(cptr);
+    }
+
+    while (copies > 0)
+    {
+      snprintf(cptr, sizeof(control) - (cptr - control), "%cdfA%03d%s\n", format,
+               getpid() % 1000, localhost);
+      cptr   += strlen(cptr);
+      copies --;
+    }
+
+    snprintf(cptr, sizeof(control) - (cptr - control),
+             "UdfA%03d%s\nN%s\n",
+             getpid() % 1000, localhost, title);
+
+    fprintf(stderr, "DEBUG: Control file is:\n%s", control);
+
+    if (order == ORDER_CONTROL_DATA)
+    {
+      lpd_command(fd, "\002%d cfA%03.3d%s\n", strlen(control), getpid() % 1000,
+        	  localhost);
+
+      fprintf(stderr, "INFO: Sending control file (%lu bytes)\n",
+              (unsigned long)strlen(control));
+
+      alarm(30);
+
+      if (lpd_write(fd, control, strlen(control) + 1) < (strlen(control) + 1))
+      {
+	status = errno;
+	perror("ERROR: Unable to write control file");
+      }
+      else
+      {
+        alarm(30);
+
+        if (read(fd, &status, 1) < 1)
+	  status = errno;
+
+        alarm(0);
+      }
+
+      if (status != 0)
+	fprintf(stderr, "ERROR: Remote host did not accept control file (%d)\n",
+        	status);
+      else
+	fputs("INFO: Control file sent successfully\n", stderr);
+    }
     else
-      fputs("INFO: Control file sent successfully\n", stderr);
+      status = 0;
+
+    if (status == 0)
+    {
+     /*
+      * Send the print file...
+      */
+
+      lpd_command(fd, "\003%u dfA%03.3d%s\n", (unsigned)filestats.st_size,
+                  getpid() % 1000, localhost);
+
+      fprintf(stderr, "INFO: Sending data file (%u bytes)\n",
+              (unsigned)filestats.st_size);
+
+      tbytes = 0;
+      for (copy = 0; copy < manual_copies; copy ++)
+      {
+	rewind(fp);
+
+	while ((nbytes = fread(buffer, 1, sizeof(buffer), fp)) > 0)
+	{
+	  fprintf(stderr, "INFO: Spooling LPR job, %u%% complete...\n",
+        	  (unsigned)(100.0f * tbytes / filestats.st_size));
+
+	  if (lpd_write(fd, buffer, nbytes) < nbytes)
+	  {
+            perror("ERROR: Unable to send print file to printer");
+            break;
+	  }
+	  else
+            tbytes += nbytes;
+	}
+      }
+
+      if (tbytes < filestats.st_size)
+	status = errno;
+      else if (lpd_write(fd, "", 1) < 1)
+	status = errno;
+      else
+      {
+        alarm(30);
+
+        if (recv(fd, &status, 1, 0) < 1)
+	  status = errno;
+
+	alarm(0);
+      }
+
+      if (status != 0)
+	fprintf(stderr, "ERROR: Remote host did not accept data file (%d)\n",
+        	status);
+      else
+	fputs("INFO: Data file sent successfully\n", stderr);
+    }
+
+    if (status == 0 && order == ORDER_DATA_CONTROL)
+    {
+      lpd_command(fd, "\002%d cfA%03.3d%s\n", strlen(control), getpid() % 1000,
+        	  localhost);
+
+      fprintf(stderr, "INFO: Sending control file (%lu bytes)\n",
+              (unsigned long)strlen(control));
+
+      alarm(30);
+
+      if (lpd_write(fd, control, strlen(control) + 1) < (strlen(control) + 1))
+      {
+	status = errno;
+	perror("ERROR: Unable to write control file");
+      }
+      else
+      {
+        alarm(30);
+
+	if (read(fd, &status, 1) < 1)
+	  status = errno;
+
+	alarm(0);
+      }
+
+      if (status != 0)
+	fprintf(stderr, "ERROR: Remote host did not accept control file (%d)\n",
+        	status);
+      else
+	fputs("INFO: Control file sent successfully\n", stderr);
+    }
+
+   /*
+    * Close the socket connection and input file...
+    */
+
+    close(fd);
+    fclose(fp);
+
+    if (status == 0)
+      return (0);
+
+   /*
+    * Restore the SIGTERM handler if we are waiting for a retry...
+    */
+
+    if (fromstdin)
+    {
+#ifdef HAVE_SIGSET /* Use System V signals over POSIX to avoid bugs */
+      sigset(SIGTERM, SIG_DFL);
+#elif defined(HAVE_SIGACTION)
+      memset(&action, 0, sizeof(action));
+
+      sigemptyset(&action.sa_mask);
+      action.sa_handler = SIG_DFL;
+      sigaction(SIGTERM, &action, NULL);
+#else
+      signal(SIGTERM, SIG_DFL);
+#endif /* HAVE_SIGSET */
+    }
+
+    sleep(30);
   }
+}
 
- /*
-  * Close the socket connection and input file and return...
-  */
 
-  close(fd);
-  fclose(fp);
-  
-  return (status);
+/*
+ * 'lpd_timeout()' - Handle timeout alarms...
+ */
+
+static void
+lpd_timeout(int sig)		/* I - Signal number */
+{
+  (void)sig;
+
+#if !defined(HAVE_SIGSET) && !defined(HAVE_SIGACTION)
+  signal(SIGALRM, lpd_timeout);
+#endif /* !HAVE_SIGSET && !HAVE_SIGACTION */
 }
 
 
@@ -760,5 +924,5 @@ rresvport(int *port)		/* IO - Port number to bind to */
 #endif /* !HAVE_RRESVPORT */
 
 /*
- * End of "$Id: lpd.c,v 1.36 2002/05/16 13:44:50 mike Exp $".
+ * End of "$Id: lpd.c,v 1.47 2002/12/17 18:56:33 swdev Exp $".
  */
