@@ -1,9 +1,9 @@
 /*
- * "$Id: lpq.c,v 1.21 2002/05/16 13:44:51 mike Exp $"
+ * "$Id: lpq.c,v 1.23 2002/12/17 18:56:35 swdev Exp $"
  *
  *   "lpq" command for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products.
+ *   Copyright 1997-2003 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -132,7 +132,18 @@ main(int  argc,		/* I - Number of command-line arguments */
 	    }
 
 	    if ((instance = strchr(dest, '/')) != NULL)
-	      *instance = '\0';
+	      *instance++ = '\0';
+
+            if (cupsGetDest(dest, instance, num_dests, dests) == NULL)
+	    {
+	      if (instance)
+		fprintf(stderr, "lpq: Unknown destination \"%s/%s\"!\n",
+		        dest, instance);
+              else
+		fprintf(stderr, "lpq: Unknown destination \"%s\"!\n", dest);
+
+	      return (1);
+	    }
 	    break;
 
 	case 'a' : /* All printers */
@@ -534,5 +545,5 @@ show_printer(http_t     *http,	/* I - HTTP connection to server */
 
 
 /*
- * End of "$Id: lpq.c,v 1.21 2002/05/16 13:44:51 mike Exp $".
+ * End of "$Id: lpq.c,v 1.23 2002/12/17 18:56:35 swdev Exp $".
  */
