@@ -1,9 +1,9 @@
 /*
- * "$Id: http.c,v 1.97 2002/05/16 13:44:53 mike Exp $"
+ * "$Id: http.c,v 1.108 2002/12/17 18:56:42 swdev Exp $"
  *
- *   HTTP routines for the Common UNIX Printing System (CUPS) scheduler.
+ *   HTTP routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -34,10 +34,6 @@
  *   httpConnectEncrypt() - Connect to a HTTP server using encryption.
  *   httpEncryption()     - Set the required encryption on the link.
  *   httpReconnect()      - Reconnect to a HTTP server...
- *   httpGetHostByName()  - Lookup a hostname or IP address, and return
- *                          address records for the specified name.
- *   httpSeparate()       - Separate a Universal Resource Identifier into its
- *                          components.
  *   httpGetSubField()    - Get a sub-field value.
  *   httpSetField()       - Set the value of an HTTP header.
  *   httpDelete()         - Send a DELETE request to the server.
@@ -52,7 +48,6 @@
  *   httpWrite()          - Write data to a HTTP connection.
  *   httpGets()           - Get a line of text from a HTTP connection.
  *   httpPrintf()         - Print a formatted string to a HTTP connection.
- *   httpStatus()         - Return a short string describing a HTTP status code.
  *   httpGetDateString()  - Get a formatted date/time string from a time value.
  *   httpGetDateTime()    - Get a time value from a formatted date/time string.
  *   httpUpdate()         - Update the current HTTP state for incoming data.
@@ -79,7 +74,6 @@
 #include <errno.h>
 
 #include "http.h"
-#include "ipp.h"
 #include "debug.h"
 
 #ifndef WIN32
@@ -185,7 +179,9 @@ void
 httpInitialize(void)
 {
 #ifdef HAVE_LIBSSL
+#  ifndef WIN32
   struct timeval	curtime;	/* Current time in microseconds */
+#  endif /* !WIN32 */
   int			i;		/* Looping var */
   unsigned char		data[1024];	/* Seed data */
 #endif /* HAVE_LIBSSL */
@@ -223,8 +219,11 @@ httpInitialize(void)
   * it is the best we can do (on others, this seed isn't even used...)
   */
 
+#ifdef WIN32
+#else
   gettimeofday(&curtime, NULL);
   srand(curtime.tv_sec + curtime.tv_usec);
+#endif /* WIN32 */
 
   for (i = 0; i < sizeof(data); i ++)
     data[i] = rand(); /* Yes, this is a poor source of random data... */
@@ -618,272 +617,6 @@ httpReconnect(http_t *http)	/* I - HTTP data */
 }
 
 
-/*
- * 'httpGetHostByName()' - Lookup a hostname or IP address, and return
- *                         address records for the specified name.
- */
-
-struct hostent *			/* O - Host entry */
-httpGetHostByName(const char *name)	/* I - Hostname or IP address */
-{
-  unsigned		ip[4];		/* IP address components */
-  static unsigned	packed_ip;	/* Packed IPv4 address */
-  static char		*packed_ptr[2];	/* Pointer to packed address */
-  static struct hostent	host_ip;	/* Host entry for IP address */
-
-#if defined(__APPLE__)
-  /* OS X hack to avoid it's ocassional long delay in lookupd */
-  static char sLoopback[] = "127.0.0.1";
-  if (strcmp(name, "localhost") == 0)
-    name = sLoopback;
-#endif /* __APPLE__ */
-
- /*
-  * This function is needed because some operating systems have a
-  * buggy implementation of httpGetHostByName() that does not support
-  * IP addresses.  If the first character of the name string is a
-  * number, then sscanf() is used to extract the IP components.
-  * We then pack the components into an IPv4 address manually,
-  * since the inet_aton() function is deprecated.  We use the
-  * htonl() macro to get the right byte order for the address.
-  */
-
-  if (isdigit(name[0]))
-  {
-   /*
-    * We have an IP address; break it up and provide the host entry
-    * to the caller.  Currently only supports IPv4 addresses, although
-    * it should be trivial to support IPv6 in CUPS 1.2.
-    */
-
-    if (sscanf(name, "%u.%u.%u.%u", ip, ip + 1, ip + 2, ip + 3) != 4)
-      return (NULL); /* Must have 4 numbers */
-
-    packed_ip = htonl(((((((ip[0] << 8) | ip[1]) << 8) | ip[2]) << 8) | ip[3]));
-
-   /*
-    * Fill in the host entry and return it...
-    */
-
-    host_ip.h_name      = (char *)name;
-    host_ip.h_aliases   = NULL;
-    host_ip.h_addrtype  = AF_INET;
-    host_ip.h_length    = 4;
-    host_ip.h_addr_list = packed_ptr;
-    packed_ptr[0]       = (char *)(&packed_ip);
-    packed_ptr[1]       = NULL;
-
-    return (&host_ip);
-  }
-  else
-  {
-   /*
-    * Use the gethostbyname() function to get the IP address for
-    * the name...
-    */
-
-    return (gethostbyname(name));
-  }
-}
-
-
-/*
- * 'httpSeparate()' - Separate a Universal Resource Identifier into its
- *                    components.
- */
-
-void
-httpSeparate(const char *uri,		/* I - Universal Resource Identifier */
-             char       *method,	/* O - Method [32] (http, https, etc.) */
-	     char       *username,	/* O - Username [32] */
-	     char       *host,		/* O - Hostname [32] */
-	     int        *port,		/* O - Port number to use */
-             char       *resource)	/* O - Resource/filename [1024] */
-{
-  char		*ptr;				/* Pointer into string... */
-  const char	*atsign,			/* @ sign */
-		*slash;				/* Separator */
-  char		safeuri[HTTP_MAX_URI];		/* "Safe" local copy of URI */
-
-
- /*
-  * Range check input...
-  */
-
-  if (uri == NULL || method == NULL || username == NULL || host == NULL ||
-      port == NULL || resource == NULL)
-    return;
-
- /*
-  * Copy the URL to a local string to make sure we don't have a URL
-  * longer than HTTP_MAX_URI characters long...
-  */
-
-  strlcpy(safeuri, uri, sizeof(safeuri));
-
-  uri = safeuri;
-
- /*
-  * Grab the method portion of the URI...
-  */
-
-  if (strncmp(uri, "//", 2) == 0)
-  {
-   /*
-    * Workaround for HP IPP client bug...
-    */
-
-    strcpy(method, "ipp");
-  }
-  else
-  {
-   /*
-    * Standard URI with method...
-    */
-
-    for (ptr = host; *uri != ':' && *uri != '\0'; uri ++)
-      if (ptr < (host + HTTP_MAX_URI - 1))
-        *ptr++ = *uri;
-
-    *ptr = '\0';
-    if (*uri == ':')
-      uri ++;
-
-   /*
-    * If the method contains a period or slash, then it's probably
-    * hostname/filename...
-    */
-
-    if (strchr(host, '.') != NULL || strchr(host, '/') != NULL || *uri == '\0')
-    {
-      if ((ptr = strchr(host, '/')) != NULL)
-      {
-	strlcpy(resource, ptr, HTTP_MAX_URI);
-	*ptr = '\0';
-      }
-      else
-	resource[0] = '\0';
-
-      if (isdigit(*uri))
-      {
-       /*
-	* OK, we have "hostname:port[/resource]"...
-	*/
-
-	*port = strtol(uri, (char **)&uri, 10);
-
-	if (*uri == '/')
-          strlcpy(resource, uri, HTTP_MAX_URI);
-      }
-      else
-	*port = 631;
-
-      strcpy(method, "http");
-      username[0] = '\0';
-      return;
-    }
-    else
-      strlcpy(method, host, 32);
-  }
-
- /*
-  * If the method starts with less than 2 slashes then it is a local resource...
-  */
-
-  if (strncmp(uri, "//", 2) != 0)
-  {
-    strlcpy(resource, uri, HTTP_MAX_URI);
-
-    username[0] = '\0';
-    host[0]     = '\0';
-    *port       = 0;
-    return;
-  }
-
- /*
-  * Grab the username, if any...
-  */
-
-  while (*uri == '/')
-    uri ++;
-
-  if ((slash = strchr(uri, '/')) == NULL)
-    slash = uri + strlen(uri);
-
-  if ((atsign = strchr(uri, '@')) != NULL && atsign < slash)
-  {
-   /*
-    * Got a username:password combo...
-    */
-
-    for (ptr = username; uri < atsign; uri ++)
-      if (ptr < (username + HTTP_MAX_URI - 1))
-	*ptr++ = *uri;
-
-    *ptr = '\0';
-
-    uri = atsign + 1;
-  }
-  else
-    username[0] = '\0';
-
- /*
-  * Grab the hostname...
-  */
-
-  for (ptr = host; *uri != ':' && *uri != '/' && *uri != '\0'; uri ++)
-    if (ptr < (host + HTTP_MAX_URI - 1))
-      *ptr++ = *uri;
-
-  *ptr = '\0';
-
-  if (*uri != ':')
-  {
-    if (strcasecmp(method, "http") == 0)
-      *port = 80;
-    else if (strcasecmp(method, "https") == 0)
-      *port = 443;
-    else if (strcasecmp(method, "ipp") == 0)
-      *port = ippPort();
-    else if (strcasecmp(method, "socket") == 0)	/* Not registered yet... */
-      *port = 9100;
-    else
-      *port = 0;
-  }
-  else
-  {
-   /*
-    * Parse port number...
-    */
-
-    *port = 0;
-    uri ++;
-    while (isdigit(*uri))
-    {
-      *port = (*port * 10) + *uri - '0';
-      uri ++;
-    }
-  }
-
-  if (*uri == '\0')
-  {
-   /*
-    * Hostname but no port or path...
-    */
-
-    resource[0] = '/';
-    resource[1] = '\0';
-    return;
-  }
-
- /*
-  * The remaining portion is the resource string...
-  */
-
-  strlcpy(resource, uri, HTTP_MAX_URI);
-}
-
-
 /*
  * 'httpGetSubField()' - Get a sub-field value.
  */
@@ -1163,11 +896,16 @@ httpRead(http_t *http,			/* I - HTTP data */
     }
 
     http->data_remaining = strtol(len, NULL, 16);
+    if (http->data_remaining < 0)
+    {
+      DEBUG_puts("httpRead: Negative chunk length!");
+      return (0);
+    }
   }
 
   DEBUG_printf(("httpRead: data_remaining = %d\n", http->data_remaining));
 
-  if (http->data_remaining == 0)
+  if (http->data_remaining <= 0)
   {
    /*
     * A zero-length chunk ends a transfer; unless we are reading POST
@@ -1219,10 +957,14 @@ httpRead(http_t *http,			/* I - HTTP data */
     {
 #ifdef WIN32
       http->error = WSAGetLastError();
-#else
-      http->error = errno;
-#endif /* WIN32 */
       return (-1);
+#else
+      if (errno != EINTR)
+      {
+        http->error = errno;
+        return (-1);
+      }
+#endif /* WIN32 */
     }
     else
       return (0);
@@ -1241,7 +983,7 @@ httpRead(http_t *http,			/* I - HTTP data */
     http->used -= length;
 
     if (http->used > 0)
-      memcpy(http->buffer, http->buffer + length, http->used);
+      memmove(http->buffer, http->buffer + length, http->used);
   }
 #ifdef HAVE_LIBSSL
   else if (http->tls)
@@ -1257,11 +999,16 @@ httpRead(http_t *http,			/* I - HTTP data */
   if (bytes > 0)
     http->data_remaining -= bytes;
   else if (bytes < 0)
+  {
 #ifdef WIN32
     http->error = WSAGetLastError();
 #else
-    http->error = errno;
+    if (errno == EINTR)
+      bytes = 0;
+    else
+      http->error = errno;
 #endif /* WIN32 */
+  }
 
   if (http->data_remaining == 0)
   {
@@ -1372,6 +1119,22 @@ httpWrite(http_t     *http,		/* I - HTTP data */
 
     if (bytes < 0)
     {
+#ifdef WIN32
+      if (WSAGetLastError() != http->error)
+      {
+        http->error = WSAGetLastError();
+	continue;
+      }
+#else
+      if (errno == EINTR)
+        continue;
+      else if (errno != http->error)
+      {
+        http->error = errno;
+	continue;
+      }
+#endif /* WIN32 */
+
       DEBUG_puts("httpWrite: error writing data...\n");
 
       return (-1);
@@ -1507,7 +1270,9 @@ httpGets(char   *line,			/* I - Line to read into */
 
         DEBUG_printf(("httpGets(): recv() error %d!\n", WSAGetLastError()));
 #else
-        if (errno != http->error)
+        if (errno == EINTR)
+	  continue;
+	else if (errno != http->error)
 	{
 	  http->error = errno;
 	  continue;
@@ -1532,6 +1297,7 @@ httpGets(char   *line,			/* I - Line to read into */
 
       http->used += bytes;
       bufend     += bytes;
+      bufptr     = bufend;
     }
   }
   while (bufptr >= bufend && http->used < HTTP_MAX_BUFFER);
@@ -1568,7 +1334,7 @@ httpGets(char   *line,			/* I - Line to read into */
 
     http->used -= bytes;
     if (http->used > 0)
-      memcpy(http->buffer, bufptr, http->used);
+      memmove(http->buffer, bufptr, http->used);
 
     DEBUG_printf(("httpGets(): Returning \"%s\"\n", line));
     return (line);
@@ -1613,60 +1379,33 @@ httpPrintf(http_t     *http,		/* I - HTTP data */
     nbytes = send(http->fd, bufptr, bytes - tbytes, 0);
 
     if (nbytes < 0)
+    {
+      nbytes = 0;
+
+#ifdef WIN32
+      if (WSAGetLastError() != http->error)
+      {
+        http->error = WSAGetLastError();
+	continue;
+      }
+#else
+      if (errno == EINTR)
+	continue;
+      else if (errno != http->error)
+      {
+        http->error = errno;
+	continue;
+      }
+#endif /* WIN32 */
+
       return (-1);
+    }
   }
 
   return (bytes);
 }
 
 
-/*
- * 'httpStatus()' - Return a short string describing a HTTP status code.
- */
-
-const char *				/* O - String or NULL */
-httpStatus(http_status_t status)	/* I - HTTP status code */
-{
-  switch (status)
-  {
-    case HTTP_CONTINUE :
-        return ("Continue");
-    case HTTP_SWITCHING_PROTOCOLS :
-        return ("Switching Protocols");
-    case HTTP_OK :
-        return ("OK");
-    case HTTP_CREATED :
-        return ("Created");
-    case HTTP_ACCEPTED :
-        return ("Accepted");
-    case HTTP_NO_CONTENT :
-        return ("No Content");
-    case HTTP_NOT_MODIFIED :
-        return ("Not Modified");
-    case HTTP_BAD_REQUEST :
-        return ("Bad Request");
-    case HTTP_UNAUTHORIZED :
-        return ("Unauthorized");
-    case HTTP_FORBIDDEN :
-        return ("Forbidden");
-    case HTTP_NOT_FOUND :
-        return ("Not Found");
-    case HTTP_REQUEST_TOO_LARGE :
-        return ("Request Entity Too Large");
-    case HTTP_URI_TOO_LONG :
-        return ("URI Too Long");
-    case HTTP_UPGRADE_REQUIRED :
-        return ("Upgrade Required");
-    case HTTP_NOT_IMPLEMENTED :
-        return ("Not Implemented");
-    case HTTP_NOT_SUPPORTED :
-        return ("Not Supported");
-    default :
-        return ("Unknown");
-  }
-}
-
-
 /*
  * 'httpGetDateString()' - Get a formatted date/time string from a time value.
  */
@@ -2269,5 +2008,5 @@ http_upgrade(http_t *http)	/* I - HTTP data */
 
 
 /*
- * End of "$Id: http.c,v 1.97 2002/05/16 13:44:53 mike Exp $".
+ * End of "$Id: http.c,v 1.108 2002/12/17 18:56:42 swdev Exp $".
  */
