@@ -1,10 +1,10 @@
 /*
- * "$Id: ipp.c,v 1.70 2002/04/06 09:35:33 mike Exp $"
+ * "$Id: ipp.c,v 1.78 2002/12/17 18:56:42 swdev Exp $"
  *
- *   Internet Printing Protocol support functions for the Common UNIX
+ *   Internet Printing Protocol object functions for the Common UNIX
  *   Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -41,15 +41,11 @@
  *   ippDateToTime()        - Convert from RFC 1903 Date/Time format to UNIX
  *                            time in seconds.
  *   ippDelete()            - Delete an IPP request.
- *   ippErrorString()       - Return a textual message for the given error
- *                            message.
  *   ippFindAttribute()     - Find a named attribute in a request...
  *   ippFindNextAttribute() - Find the next named attribute in a request...
  *   ippLength()            - Compute the length of an IPP request.
  *   ippNew()               - Allocate a new IPP request.
- *   ippPort()              - Return the default IPP port number.
  *   ippRead()              - Read data for an IPP request.
- *   ippSetPort()           - Set the default port number.
  *   ippTimeToDate()        - Convert from UNIX time to RFC 1903 format.
  *   ippWrite()             - Write data for an IPP request.
  *   _ipp_add_attr()        - Add a new attribute to the request.
@@ -71,13 +67,6 @@
 #include <ctype.h>
 
 
-/*
- * Local globals...
- */
-
-static int	ipp_port = 0;
-
-
 /*
  * Local functions...
  */
@@ -620,90 +609,6 @@ ippDelete(ipp_t *ipp)		/* I - IPP request */
 }
 
 
-/*
- * 'ippErrorString()' - Return a textual message for the given error message.
- */
-
-const char *				/* O - Text string */
-ippErrorString(ipp_status_t error)	/* I - Error status */
-{
-  static char	unknown[255];		/* Unknown error statuses */
-  static const char *status_oks[] =	/* "OK" status codes */
-		{
-		  "successful-ok",
-		  "successful-ok-ignored-or-substituted-attributes",
-		  "successful-ok-conflicting-attributes",
-		  "successful-ok-ignored-subscriptions",
-		  "successful-ok-ignored-notifications",
-		  "successful-ok-too-many-events",
-		  "successful-ok-but-cancel-subscription"
-		},
-		*status_400s[] =	/* Client errors */
-		{
-		  "client-error-bad-request",
-		  "client-error-forbidden",
-		  "client-error-not-authenticated",
-		  "client-error-not-authorized",
-		  "client-error-not-possible",
-		  "client-error-timeout",
-		  "client-error-not-found",
-		  "client-error-gone",
-		  "client-error-request-entity-too-large",
-		  "client-error-request-value-too-long",
-		  "client-error-document-format-not-supported",
-		  "client-error-attributes-or-values-not-supported",
-		  "client-error-uri-scheme-not-supported",
-		  "client-error-charset-not-supported",
-		  "client-error-conflicting-attributes",
-		  "client-error-compression-not-supported",
-		  "client-error-compression-error",
-		  "client-error-document-format-error",
-		  "client-error-document-access-error",
-		  "client-error-attributes-not-settable",
-		  "client-error-ignored-all-subscriptions",
-		  "client-error-too-many-subscriptions",
-		  "client-error-ignored-all-notifications",
-		  "client-error-print-support-file-not-found"
-		},
-		*status_500s[] =	/* Server errors */
-		{
-		  "server-error-internal-error",
-		  "server-error-operation-not-supported",
-		  "server-error-service-unavailable",
-		  "server-error-version-not-supported",
-		  "server-error-device-error",
-		  "server-error-temporary-error",
-		  "server-error-not-accepting-jobs",
-		  "server-error-busy",
-		  "server-error-job-canceled",
-		  "server-error-multiple-document-jobs-not-supported",
-		  "server-error-printer-is-deactivated"
-		};
-
-
- /*
-  * See if the error code is a known value...
-  */
-
-  if (error >= IPP_OK && error <= IPP_OK_BUT_CANCEL_SUBSCRIPTION)
-    return (status_oks[error]);
-  else if (error == IPP_REDIRECTION_OTHER_SITE)
-    return ("redirection-other-site");
-  else if (error >= IPP_BAD_REQUEST && error <= IPP_PRINT_SUPPORT_FILE_NOT_FOUND)
-    return (status_400s[error - IPP_BAD_REQUEST]);
-  else if (error >= IPP_INTERNAL_ERROR && error <= IPP_PRINTER_IS_DEACTIVATED)
-    return (status_500s[error - IPP_INTERNAL_ERROR]);
-
- /*
-  * No, build an "unknown-xxxx" error string...
-  */
-
-  sprintf(unknown, "unknown-%04x", error);
-
-  return (unknown);
-}
-
-
 /*
  * 'ippFindAttribute()' - Find a named attribute in a request...
  */
@@ -850,7 +755,8 @@ ippLength(ipp_t *ipp)		/* I - IPP request */
 	  for (i = 0, value = attr->values;
 	       i < attr->num_values;
 	       i ++, value ++)
-	    bytes += strlen(value->string.text);
+	    if (value->string.text != NULL)
+	      bytes += strlen(value->string.text);
 	  break;
 
       case IPP_TAG_DATE :
@@ -868,11 +774,17 @@ ippLength(ipp_t *ipp)		/* I - IPP request */
       case IPP_TAG_TEXTLANG :
       case IPP_TAG_NAMELANG :
           bytes += 4 * attr->num_values;/* Charset + text length */
+
 	  for (i = 0, value = attr->values;
 	       i < attr->num_values;
 	       i ++, value ++)
-	    bytes += strlen(value->string.charset) +
-	             strlen(value->string.text);
+	  {
+	    if (value->string.charset != NULL)
+	      bytes += strlen(value->string.charset);
+
+	    if (value->string.text != NULL)
+	      bytes += strlen(value->string.text);
+	  }
 	  break;
 
       default :
@@ -1178,7 +1090,7 @@ ippRead(http_t *http,		/* I - HTTP data */
 	    case IPP_TAG_MIMETYPE :
                 value->string.text = calloc(n + 1, 1);
 
-	        if (ipp_read(http, value->string.text, n) < n)
+	        if (ipp_read(http, (unsigned char *)value->string.text, n) < n)
 		  return (IPP_ERROR);
 
 		DEBUG_printf(("ippRead: value = \'%s\'\n",
@@ -1343,7 +1255,7 @@ ippWrite(http_t *http,		/* I - HTTP data */
 {
   int			i;		/* Looping var */
   int			n;		/* Length of data */
-  unsigned char		buffer[8192],	/* Data buffer */
+  unsigned char		buffer[32768],	/* Data buffer */
 			*bufptr;	/* Pointer into buffer */
   ipp_attribute_t	*attr;		/* Current attribute */
   ipp_value_t		*value;		/* Current value */
@@ -1359,7 +1271,12 @@ ippWrite(http_t *http,		/* I - HTTP data */
 
     case IPP_HEADER :
        /*
-        * Send the request header...
+        * Send the request header:
+	*
+	*                 Version = 2 bytes
+	*   Operation/Status Code = 2 bytes
+	*              Request ID = 4 bytes
+	*                   Total = 8 bytes
 	*/
 
         bufptr = buffer;
@@ -1379,6 +1296,11 @@ ippWrite(http_t *http,		/* I - HTTP data */
 	  return (IPP_ERROR);
 	}
 
+       /*
+        * Reset the state engine to point to the first attribute
+	* in the request/response, with no current group.
+	*/
+
         ipp->state   = IPP_ATTRIBUTE;
 	ipp->current = ipp->attrs;
 	ipp->curtag  = IPP_TAG_ZERO;
@@ -1409,7 +1331,7 @@ ippWrite(http_t *http,		/* I - HTTP data */
           if (ipp->curtag != attr->group_tag)
 	  {
 	   /*
-	    * Send a group operation tag...
+	    * Send a group tag byte...
 	    */
 
 	    ipp->curtag = attr->group_tag;
@@ -1421,18 +1343,36 @@ ippWrite(http_t *http,		/* I - HTTP data */
 	    *bufptr++ = attr->group_tag;
 	  }
 
-          if ((n = strlen(attr->name)) > (sizeof(buffer) - 3))
+         /*
+	  * Get the length of the attribute name, and make sure it won't
+	  * overflow the buffer...
+	  */
+
+          if ((n = strlen(attr->name)) > (sizeof(buffer) - 4))
 	    return (IPP_ERROR);
 
           DEBUG_printf(("ippWrite: writing value tag = %x\n", attr->value_tag));
           DEBUG_printf(("ippWrite: writing name = %d, \'%s\'\n", n, attr->name));
 
+         /*
+	  * Write the attribute tag and name.  The current implementation
+	  * does not support the extension value tags above 0x7f, so all
+	  * value tags are 1 byte.
+	  *
+	  * The attribute name length does not include the trailing nul
+	  * character in the source string.
+	  */
+
           *bufptr++ = attr->value_tag;
 	  *bufptr++ = n >> 8;
 	  *bufptr++ = n;
 	  memcpy(bufptr, attr->name, n);
 	  bufptr += n;
 
+         /*
+	  * Now write the attribute value(s)...
+	  */
+
 	  switch (attr->value_tag & ~IPP_TAG_COPY)
 	  {
 	    case IPP_TAG_INTEGER :
@@ -1464,6 +1404,13 @@ ippWrite(http_t *http,		/* I - HTTP data */
 		    *bufptr++ = 0;
 		  }
 
+		 /*
+	          * Integers and enumerations are both 4-byte signed
+		  * (twos-complement) values.
+		  *
+		  * Put the 2-byte length and 4-byte value into the buffer...
+		  */
+
 	          *bufptr++ = 0;
 		  *bufptr++ = 4;
 		  *bufptr++ = value->integer >> 24;
@@ -1501,6 +1448,12 @@ ippWrite(http_t *http,		/* I - HTTP data */
 		    *bufptr++ = 0;
 		  }
 
+                 /*
+		  * Boolean values are 1-byte; 0 = false, 1 = true.
+		  *
+		  * Put the 2-byte length and 1-byte value into the buffer...
+		  */
+
 	          *bufptr++ = 0;
 		  *bufptr++ = 1;
 		  *bufptr++ = value->boolean;
@@ -1547,9 +1500,20 @@ ippWrite(http_t *http,		/* I - HTTP data */
 		    *bufptr++ = 0;
 		  }
 
-                  n = strlen(value->string.text);
+		 /*
+		  * All simple strings consist of the 2-byte length and
+		  * character data without the trailing nul normally found
+		  * in C strings.  Also, strings cannot be longer than 32767
+		  * bytes since the 2-byte length is a signed (twos-complement)
+		  * value.
+		  */
 
-                  if (n > sizeof(buffer))
+                  if (value->string.text != NULL)
+                    n = strlen(value->string.text);
+		  else
+		    n = 0;
+
+                  if (n > (sizeof(buffer) - 2))
 		    return (IPP_ERROR);
 
                   DEBUG_printf(("ippWrite: writing string = %d, \'%s\'\n", n,
@@ -1566,10 +1530,19 @@ ippWrite(http_t *http,		/* I - HTTP data */
 		    bufptr = buffer;
 		  }
 
+                 /*
+		  * Put the 2-byte length and string characters in the
+		  * buffer.
+		  */
+
 	          *bufptr++ = n >> 8;
 		  *bufptr++ = n;
-		  memcpy(bufptr, value->string.text, n);
-		  bufptr += n;
+
+		  if (n > 0)
+		  {
+		    memcpy(bufptr, value->string.text, n);
+		    bufptr += n;
+		  }
 		}
 		break;
 
@@ -1601,6 +1574,14 @@ ippWrite(http_t *http,		/* I - HTTP data */
 		    *bufptr++ = 0;
 		  }
 
+                 /*
+		  * Date values consist of a 2-byte length and an
+		  * 11-byte date/time structure defined by RFC 1903.
+		  *
+		  * Put the 2-byte length and 11-byte date/time
+		  * structure in the buffer.
+		  */
+
 	          *bufptr++ = 0;
 		  *bufptr++ = 11;
 		  memcpy(bufptr, value->date, 11);
@@ -1636,6 +1617,15 @@ ippWrite(http_t *http,		/* I - HTTP data */
 		    *bufptr++ = 0;
 		  }
 
+                 /*
+		  * Resolution values consist of a 2-byte length,
+		  * 4-byte horizontal resolution value, 4-byte vertical
+		  * resolution value, and a 1-byte units value.
+		  *
+		  * Put the 2-byte length and resolution value data
+		  * into the buffer.
+		  */
+
 	          *bufptr++ = 0;
 		  *bufptr++ = 9;
 		  *bufptr++ = value->resolution.xres >> 24;
@@ -1678,6 +1668,14 @@ ippWrite(http_t *http,		/* I - HTTP data */
 		    *bufptr++ = 0;
 		  }
 
+                 /*
+		  * Range values consist of a 2-byte length,
+		  * 4-byte lower value, and 4-byte upper value.
+		  *
+		  * Put the 2-byte length and range value data
+		  * into the buffer.
+		  */
+
 	          *bufptr++ = 0;
 		  *bufptr++ = 8;
 		  *bufptr++ = value->range.lower >> 24;
@@ -1720,11 +1718,25 @@ ippWrite(http_t *http,		/* I - HTTP data */
 		    *bufptr++ = 0;
 		  }
 
-                  n = strlen(value->string.charset) +
-		      strlen(value->string.text) +
-		      4;
+                 /*
+		  * textWithLanguage and nameWithLanguage values consist
+		  * of a 2-byte length for both strings and their
+		  * individual lengths, a 2-byte length for the
+		  * character string, the character string without the
+		  * trailing nul, a 2-byte length for the character
+		  * set string, and the character set string without
+		  * the trailing nul.
+		  */
 
-                  if (n > sizeof(buffer))
+                  n = 4;
+
+		  if (value->string.charset != NULL)
+                    n += strlen(value->string.charset);
+
+		  if (value->string.text != NULL)
+                    n += strlen(value->string.text);
+
+                  if (n > (sizeof(buffer) - 2))
 		    return (IPP_ERROR);
 
                   if ((sizeof(buffer) - (bufptr - buffer)) < (n + 2))
@@ -1743,22 +1755,36 @@ ippWrite(http_t *http,		/* I - HTTP data */
 		  *bufptr++ = n;
 
                  /* Length of charset */
-                  n = strlen(value->string.charset);
+		  if (value->string.charset != NULL)
+		    n = strlen(value->string.charset);
+		  else
+		    n = 0;
+
 	          *bufptr++ = n >> 8;
 		  *bufptr++ = n;
 
                  /* Charset */
-		  memcpy(bufptr, value->string.charset, n);
-		  bufptr += n;
+		  if (n > 0)
+		  {
+		    memcpy(bufptr, value->string.charset, n);
+		    bufptr += n;
+		  }
 
                  /* Length of text */
-                  n = strlen(value->string.text);
+                  if (value->string.text != NULL)
+		    n = strlen(value->string.text);
+		  else
+		    n = 0;
+
 	          *bufptr++ = n >> 8;
 		  *bufptr++ = n;
 
                  /* Text */
-		  memcpy(bufptr, value->string.text, n);
-		  bufptr += n;
+		  if (n > 0)
+		  {
+		    memcpy(bufptr, value->string.text, n);
+		    bufptr += n;
+		  }
 		}
 		break;
 
@@ -1790,9 +1816,16 @@ ippWrite(http_t *http,		/* I - HTTP data */
 		    *bufptr++ = 0;
 		  }
 
+                 /*
+		  * An unknown value might just be a collection value,
+		  * or some new value that a vendor has come up with.
+		  * It consists of a 2-byte length and the bytes in
+		  * the unknown value buffer.
+		  */
+
                   n = value->unknown.length;
 
-                  if (n > sizeof(buffer))
+                  if (n > (sizeof(buffer) - 2))
 		    return (IPP_ERROR);
 
                   if ((sizeof(buffer) - (bufptr - buffer)) < (n + 2))
@@ -1868,39 +1901,6 @@ ippWrite(http_t *http,		/* I - HTTP data */
 }
 
 
-/*
- * 'ippPort()' - Return the default IPP port number.
- */
-
-int				/* O - Port number */
-ippPort(void)
-{
-  const char	*server_port;	/* SERVER_PORT environment variable */
-  struct servent *port;		/* Port number info */  
-
-
-  if (ipp_port)
-    return (ipp_port);
-  else if ((server_port = getenv("IPP_PORT")) != NULL)
-    return (ipp_port = atoi(server_port));
-  else if ((port = getservbyname("ipp", NULL)) == NULL)
-    return (ipp_port = IPP_PORT);
-  else
-    return (ipp_port = ntohs(port->s_port));
-}
-
-
-/*
- * 'ippSetPort()' - Set the default port number.
- */
-
-void
-ippSetPort(int p)		/* I - Port number to use */
-{
-  ipp_port = p;
-}
-
-
 /*
  * '_ipp_add_attr()' - Add a new attribute to the request.
  */
@@ -2031,7 +2031,7 @@ ipp_read(http_t        *http,	/* I - Client connection */
       http->data_remaining -= bytes;
 
       if (http->used > 0)
-	memcpy(http->buffer, http->buffer + bytes, http->used);
+	memmove(http->buffer, http->buffer + bytes, http->used);
 
       if (http->data_remaining == 0)
       {
@@ -2047,7 +2047,7 @@ ipp_read(http_t        *http,	/* I - Client connection */
 	}
       }
     }
-    else if ((bytes = httpRead(http, (char *)buffer, length - tbytes)) <= 0)
+    else if ((bytes = httpRead(http, (char *)buffer, length - tbytes)) < 0)
       break;
   }
 
@@ -2063,5 +2063,5 @@ ipp_read(http_t        *http,	/* I - Client connection */
 
 
 /*
- * End of "$Id: ipp.c,v 1.70 2002/04/06 09:35:33 mike Exp $".
+ * End of "$Id: ipp.c,v 1.78 2002/12/17 18:56:42 swdev Exp $".
  */
