@@ -1,9 +1,9 @@
 /*
- * "$Id: ppd.c,v 1.70 2002/05/31 19:07:18 mike Exp $"
+ * "$Id: ppd.c,v 1.75 2002/12/17 18:56:43 swdev Exp $"
  *
  *   PPD file routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -45,7 +45,6 @@
  *   compare_strings() - Compare two strings.
  *   compare_groups()  - Compare two groups.
  *   compare_options() - Compare two options.
- *   compare_choices() - Compare two choices.
  */
 
 /*
@@ -84,14 +83,19 @@
  * Local functions...
  */
 
+#ifndef __APPLE__
 static int		compare_strings(char *s, char *t);
 static int		compare_groups(ppd_group_t *g0, ppd_group_t *g1);
 static int		compare_options(ppd_option_t *o0, ppd_option_t *o1);
-static int		compare_choices(ppd_choice_t *c0, ppd_choice_t *c1);
+#endif /* !__APPLE__ */
 static int		ppd_read(FILE *fp, char *keyword, char *option,
 			         char *text, char **string);
 static void		ppd_decode(char *string);
+#ifdef __APPLE__
+#  define ppd_fix(s)
+#else
 static void		ppd_fix(char *string);
+#endif /* __APPLE__ */
 static void		ppd_free_group(ppd_group_t *group);
 static void		ppd_free_option(ppd_option_t *option);
 static ppd_group_t	*ppd_get_group(ppd_file_t *ppd, char *name);
@@ -424,6 +428,7 @@ ppd_add_size(ppd_file_t *ppd,	/* I - PPD file */
 ppd_file_t *			/* O - PPD file record */
 ppdOpen(FILE *fp)		/* I - File to read from */
 {
+  char		*oldlocale;	/* Old locale settings */
   int		i, j, k, m;	/* Looping vars */
   int		count;		/* Temporary count */
   ppd_file_t	*ppd;		/* PPD file record */
@@ -452,12 +457,6 @@ ppdOpen(FILE *fp)		/* I - File to read from */
   cups_lang_t	*language;	/* Default language */
 
 
- /*
-  * Get the default language for the user...
-  */
-
-  language = cupsLangDefault();
-
  /*
   * Range check input...
   */
@@ -500,6 +499,18 @@ ppdOpen(FILE *fp)		/* I - File to read from */
   ppd->colorspace     = PPD_CS_GRAY;
   ppd->landscape      = -90;
 
+ /*
+  * Get the default language for the user...
+  */
+
+  language = cupsLangDefault();
+
+#ifdef LC_NUMERIC
+  oldlocale = setlocale(LC_NUMERIC, "C");
+#else
+  oldlocale = setlocale(LC_ALL, "C");
+#endif /* LC_NUMERIC */
+
  /*
   * Read lines from the PPD file and add them to the file record...
   */
@@ -543,6 +554,15 @@ ppdOpen(FILE *fp)		/* I - File to read from */
       */
 
       ppdClose(ppd);
+
+      cupsLangFree(language);
+
+#ifdef LC_NUMERIC
+      setlocale(LC_NUMERIC, oldlocale);
+#else
+      setlocale(LC_ALL, oldlocale);
+#endif /* LC_NUMERIC */
+
       return (NULL);
     }
 
@@ -666,7 +686,17 @@ ppdOpen(FILE *fp)		/* I - File to read from */
       if (filter == NULL)
       {
         safe_free(filter);
+
 	ppdClose(ppd);
+
+        cupsLangFree(language);
+
+#ifdef LC_NUMERIC
+        setlocale(LC_NUMERIC, oldlocale);
+#else
+        setlocale(LC_ALL, oldlocale);
+#endif /* LC_NUMERIC */
+
 	return (NULL);
       }
 
@@ -698,7 +728,17 @@ ppdOpen(FILE *fp)		/* I - File to read from */
       if (tempfonts == NULL)
       {
         safe_free(string);
+
         ppdClose(ppd);
+
+        cupsLangFree(language);
+
+#ifdef LC_NUMERIC
+        setlocale(LC_NUMERIC, oldlocale);
+#else
+        setlocale(LC_ALL, oldlocale);
+#endif /* LC_NUMERIC */
+
 	return (NULL);
       }
       
@@ -732,14 +772,34 @@ ppdOpen(FILE *fp)		/* I - File to read from */
                                                  CUPS_MSG_GENERAL))) == NULL)
 	{
           ppdClose(ppd);
+
 	  safe_free(string);
+
+          cupsLangFree(language);
+
+#ifdef LC_NUMERIC
+          setlocale(LC_NUMERIC, oldlocale);
+#else
+          setlocale(LC_ALL, oldlocale);
+#endif /* LC_NUMERIC */
+
 	  return (NULL);
 	}
 
 	if ((option = ppd_get_option(temp, "PageSize")) == NULL)
 	{
           ppdClose(ppd);
+
 	  safe_free(string);
+
+          cupsLangFree(language);
+
+#ifdef LC_NUMERIC
+          setlocale(LC_NUMERIC, oldlocale);
+#else
+          setlocale(LC_ALL, oldlocale);
+#endif /* LC_NUMERIC */
+
 	  return (NULL);
 	}
       }
@@ -747,7 +807,17 @@ ppdOpen(FILE *fp)		/* I - File to read from */
       if ((choice = ppd_add_choice(option, "Custom")) == NULL)
       {
         ppdClose(ppd);
+
 	safe_free(string);
+
+        cupsLangFree(language);
+
+#ifdef LC_NUMERIC
+        setlocale(LC_NUMERIC, oldlocale);
+#else
+        setlocale(LC_ALL, oldlocale);
+#endif /* LC_NUMERIC */
+
 	return (NULL);
       }
 
@@ -799,16 +869,38 @@ ppdOpen(FILE *fp)		/* I - File to read from */
                                                    CUPS_MSG_GENERAL))) == NULL)
 	  {
 	    DEBUG_puts("Unable to get general group!");
+
             ppdClose(ppd);
+
 	    safe_free(string);
+
+            cupsLangFree(language);
+
+#ifdef LC_NUMERIC
+            setlocale(LC_NUMERIC, oldlocale);
+#else
+            setlocale(LC_ALL, oldlocale);
+#endif /* LC_NUMERIC */
+
 	    return (NULL);
 	  }
 
 	  if ((option = ppd_get_option(temp, "PageSize")) == NULL)
 	  {
 	    DEBUG_puts("Unable to get PageSize option!");
+
             ppdClose(ppd);
+
 	    safe_free(string);
+
+            cupsLangFree(language);
+
+#ifdef LC_NUMERIC
+            setlocale(LC_NUMERIC, oldlocale);
+#else
+            setlocale(LC_ALL, oldlocale);
+#endif /* LC_NUMERIC */
+
 	    return (NULL);
 	  }
         }
@@ -816,8 +908,19 @@ ppdOpen(FILE *fp)		/* I - File to read from */
 	if ((choice = ppd_add_choice(option, "Custom")) == NULL)
 	{
 	  DEBUG_puts("Unable to add Custom choice!");
+
           ppdClose(ppd);
+
 	  safe_free(string);
+
+          cupsLangFree(language);
+
+#ifdef LC_NUMERIC
+          setlocale(LC_NUMERIC, oldlocale);
+#else
+          setlocale(LC_ALL, oldlocale);
+#endif /* LC_NUMERIC */
+
 	  return (NULL);
 	}
 
@@ -829,16 +932,38 @@ ppdOpen(FILE *fp)		/* I - File to read from */
       if ((option = ppdFindOption(ppd, "PageSize")) == NULL)
       {
 	DEBUG_puts("Unable to find PageSize option!");
-	ppdClose(ppd);
+
+        ppdClose(ppd);
+
 	safe_free(string);
+
+        cupsLangFree(language);
+
+#ifdef LC_NUMERIC
+        setlocale(LC_NUMERIC, oldlocale);
+#else
+        setlocale(LC_ALL, oldlocale);
+#endif /* LC_NUMERIC */
+
 	return (NULL);
       }
 
       if ((choice = ppdFindChoice(option, "Custom")) == NULL)
       {
 	DEBUG_puts("Unable to find Custom choice!");
+
         ppdClose(ppd);
+
 	safe_free(string);
+
+        cupsLangFree(language);
+
+#ifdef LC_NUMERIC
+        setlocale(LC_NUMERIC, oldlocale);
+#else
+        setlocale(LC_ALL, oldlocale);
+#endif /* LC_NUMERIC */
+
 	return (NULL);
       }
 
@@ -915,8 +1040,18 @@ ppdOpen(FILE *fp)		/* I - File to read from */
 	                             strlen(string) + 1);
         if (temp == NULL)
 	{
+          ppdClose(ppd);
+
 	  safe_free(string);
-	  ppdClose(ppd);
+
+          cupsLangFree(language);
+
+#ifdef LC_NUMERIC
+          setlocale(LC_NUMERIC, oldlocale);
+#else
+          setlocale(LC_ALL, oldlocale);
+#endif /* LC_NUMERIC */
+
 	  return (NULL);
 	}
 
@@ -962,8 +1097,18 @@ ppdOpen(FILE *fp)		/* I - File to read from */
 
         if (group == NULL)
 	{
-	  ppdClose(ppd);
+          ppdClose(ppd);
+
 	  safe_free(string);
+
+          cupsLangFree(language);
+
+#ifdef LC_NUMERIC
+          setlocale(LC_NUMERIC, oldlocale);
+#else
+          setlocale(LC_ALL, oldlocale);
+#endif /* LC_NUMERIC */
+
 	  return (NULL);
 	}
 
@@ -976,8 +1121,18 @@ ppdOpen(FILE *fp)		/* I - File to read from */
 
       if (option == NULL)
       {
-	ppdClose(ppd);
+        ppdClose(ppd);
+
 	safe_free(string);
+
+        cupsLangFree(language);
+
+#ifdef LC_NUMERIC
+        setlocale(LC_NUMERIC, oldlocale);
+#else
+        setlocale(LC_ALL, oldlocale);
+#endif /* LC_NUMERIC */
+
 	return (NULL);
       }
 
@@ -1031,7 +1186,17 @@ ppdOpen(FILE *fp)		/* I - File to read from */
       if (group == NULL)
       {
         ppdClose(ppd);
+
 	safe_free(string);
+
+        cupsLangFree(language);
+
+#ifdef LC_NUMERIC
+        setlocale(LC_NUMERIC, oldlocale);
+#else
+        setlocale(LC_ALL, oldlocale);
+#endif /* LC_NUMERIC */
+
 	return (NULL);
       }
 
@@ -1047,7 +1212,17 @@ ppdOpen(FILE *fp)		/* I - File to read from */
       if (option == NULL)
       {
         ppdClose(ppd);
+
 	safe_free(string);
+
+        cupsLangFree(language);
+
+#ifdef LC_NUMERIC
+        setlocale(LC_NUMERIC, oldlocale);
+#else
+        setlocale(LC_ALL, oldlocale);
+#endif /* LC_NUMERIC */
+
 	return (NULL);
       }
 
@@ -1079,7 +1254,17 @@ ppdOpen(FILE *fp)		/* I - File to read from */
       if (group != NULL)
       {
         ppdClose(ppd);
+
 	safe_free(string);
+
+        cupsLangFree(language);
+
+#ifdef LC_NUMERIC
+        setlocale(LC_NUMERIC, oldlocale);
+#else
+        setlocale(LC_ALL, oldlocale);
+#endif /* LC_NUMERIC */
+
 	return (NULL);
       }
 
@@ -1119,7 +1304,17 @@ ppdOpen(FILE *fp)		/* I - File to read from */
       if (sscanf(string, "%f%40s%40s", &order, name, keyword) != 3)
       {
         ppdClose(ppd);
+
 	safe_free(string);
+
+        cupsLangFree(language);
+
+#ifdef LC_NUMERIC
+        setlocale(LC_NUMERIC, oldlocale);
+#else
+        setlocale(LC_ALL, oldlocale);
+#endif /* LC_NUMERIC */
+
 	return (NULL);
       }
 
@@ -1211,8 +1406,18 @@ ppdOpen(FILE *fp)		/* I - File to read from */
 
       if (constraint == NULL)
       {
-	ppdClose(ppd);
+        ppdClose(ppd);
+
 	safe_free(string);
+
+        cupsLangFree(language);
+
+#ifdef LC_NUMERIC
+        setlocale(LC_NUMERIC, oldlocale);
+#else
+        setlocale(LC_ALL, oldlocale);
+#endif /* LC_NUMERIC */
+
 	return (NULL);
       }
 
@@ -1293,7 +1498,17 @@ ppdOpen(FILE *fp)		/* I - File to read from */
 	*/
 
         ppdClose(ppd);
+
 	safe_free(string);
+
+        cupsLangFree(language);
+
+#ifdef LC_NUMERIC
+        setlocale(LC_NUMERIC, oldlocale);
+#else
+        setlocale(LC_ALL, oldlocale);
+#endif /* LC_NUMERIC */
+
 	return (NULL);
       }
 
@@ -1311,7 +1526,17 @@ ppdOpen(FILE *fp)		/* I - File to read from */
 	*/
 
         ppdClose(ppd);
+
 	safe_free(string);
+
+        cupsLangFree(language);
+
+#ifdef LC_NUMERIC
+        setlocale(LC_NUMERIC, oldlocale);
+#else
+        setlocale(LC_ALL, oldlocale);
+#endif /* LC_NUMERIC */
+
 	return (NULL);
       }
 
@@ -1368,6 +1593,7 @@ ppdOpen(FILE *fp)		/* I - File to read from */
     printf("Premature EOF at %lu...\n", (unsigned long)ftell(fp));
 #endif /* DEBUG */
 
+#ifndef __APPLE__
  /*
   * Make sure that all PPD files with an InputSlot option have an
   * "auto" choice that maps to no specific tray or media type.
@@ -1392,55 +1618,70 @@ ppdOpen(FILE *fp)		/* I - File to read from */
       choice->code = NULL;
     }
   }
+#endif /* !__APPLE__ */
 
  /*
   * Set the option back-pointer for each choice...
   */
 
+#ifndef __APPLE__
   qsort(ppd->groups, ppd->num_groups, sizeof(ppd_group_t),
         (int (*)(const void *, const void *))compare_groups);
+#endif /* !__APPLE__ */
 
   for (i = ppd->num_groups, group = ppd->groups;
        i > 0;
        i --, group ++)
   {
+#ifndef __APPLE__
     qsort(group->options, group->num_options, sizeof(ppd_option_t),
           (int (*)(const void *, const void *))compare_options);
+#endif /* !__APPLE__ */
 
     for (j = group->num_options, option = group->options;
          j > 0;
 	 j --, option ++)
     {
-      qsort(option->choices, option->num_choices, sizeof(ppd_choice_t),
-            (int (*)(const void *, const void *))compare_choices);
-
       for (k = 0; k < option->num_choices; k ++)
         option->choices[k].option = (void *)option;
     }
 
+#ifndef __APPLE__
     qsort(group->subgroups, group->num_subgroups, sizeof(ppd_group_t),
           (int (*)(const void *, const void *))compare_groups);
+#endif /* !__APPLE__ */
 
     for (j = group->num_subgroups, subgroup = group->subgroups;
          j > 0;
 	 j --, subgroup ++)
     {
+#ifndef __APPLE__
       qsort(subgroup->options, subgroup->num_options, sizeof(ppd_option_t),
             (int (*)(const void *, const void *))compare_options);
+#endif /* !__APPLE__ */
 
       for (k = group->num_options, option = group->options;
            k > 0;
 	   k --, option ++)
       {
-	qsort(option->choices, option->num_choices, sizeof(ppd_choice_t),
-              (int (*)(const void *, const void *))compare_choices);
-
         for (m = 0; m < option->num_choices; m ++)
           option->choices[m].option = (void *)option;
       }
     }
   }
 
+ /*
+  * Reset language preferences...
+  */
+
+  cupsLangFree(language);
+
+#ifdef LC_NUMERIC
+  setlocale(LC_NUMERIC, oldlocale);
+#else
+  setlocale(LC_ALL, oldlocale);
+#endif /* LC_NUMERIC */
+
   return (ppd);
 }
 
@@ -1517,6 +1758,7 @@ ppdOpenFile(const char *filename) /* I - File to read from */
 }
 
 
+#ifndef __APPLE__
 /*
  * 'compare_strings()' - Compare two strings.
  */
@@ -1648,18 +1890,7 @@ compare_options(ppd_option_t *o0,/* I - First option */
 {
   return (compare_strings(o0->text, o1->text));
 }
-
-
-/*
- * 'compare_choices()' - Compare two choices.
- */
-
-static int			/* O - Result of comparison */
-compare_choices(ppd_choice_t *c0,/* I - First choice */
-                ppd_choice_t *c1)/* I - Second choice */
-{
-  return (compare_strings(c0->text, c1->text));
-}
+#endif /* !__APPLE__ */
 
 
 /*
@@ -2001,6 +2232,7 @@ ppd_decode(char *string)	/* I - String to decode */
 }
 
 
+#ifndef __APPLE__
 /*
  * 'ppd_fix()' - Fix WinANSI characters in the range 0x80 to 0x9f to be
  *               valid ISO-8859-1 characters...
@@ -2051,8 +2283,9 @@ ppd_fix(char *string)		/* IO - String to fix */
     if (*p >= 0x80 && *p < 0xa0)
       *p = lut[*p - 0x80];
 }
+#endif /* !__APPLE__ */
 
 
 /*
- * End of "$Id: ppd.c,v 1.70 2002/05/31 19:07:18 mike Exp $".
+ * End of "$Id: ppd.c,v 1.75 2002/12/17 18:56:43 swdev Exp $".
  */
