@@ -1,9 +1,9 @@
 /*
- * "$Id: util.c,v 1.96 2002/05/31 13:02:19 mike Exp $"
+ * "$Id: util.c,v 1.103 2002/12/19 15:08:46 mike Exp $"
  *
  *   Printing utilities for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products.
+ *   Copyright 1997-2003 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -195,7 +195,7 @@ cupsDoFileRequest(http_t     *http,	/* I - HTTP connection to server */
     return (NULL);
   }
 
-  DEBUG_printf(("cupsDoFileRequest(%p, %08x, \'%s\', \'%s\')\n",
+  DEBUG_printf(("cupsDoFileRequest(%p, %p, \'%s\', \'%s\')\n",
                 http, request, resource, filename ? filename : "(null)"));
 
  /*
@@ -215,7 +215,11 @@ cupsDoFileRequest(http_t     *http,	/* I - HTTP connection to server */
       return (NULL);
     }
 
+#ifdef WIN32
+    if (fileinfo.st_mode & _S_IFDIR)
+#else
     if (S_ISDIR(fileinfo.st_mode))
+#endif /* WIN32 */
     {
      /*
       * Can't send a directory...
@@ -267,6 +271,8 @@ cupsDoFileRequest(http_t     *http,	/* I - HTTP connection to server */
     httpSetField(http, HTTP_FIELD_CONTENT_TYPE, "application/ipp");
     httpSetField(http, HTTP_FIELD_AUTHORIZATION, authstring);
 
+    DEBUG_printf(("cupsDoFileRequest: authstring=\"%s\"\n", authstring));
+
    /*
     * Try the request...
     */
@@ -315,6 +321,8 @@ cupsDoFileRequest(http_t     *http,	/* I - HTTP connection to server */
 
     while ((status = httpUpdate(http)) == HTTP_CONTINUE);
 
+    DEBUG_printf(("cupsDoFileRequest: status = %d\n", status));
+
     if (status == HTTP_UNAUTHORIZED)
     {
       DEBUG_puts("cupsDoFileRequest: unauthorized...");
@@ -324,6 +332,7 @@ cupsDoFileRequest(http_t     *http,	/* I - HTTP connection to server */
       */
 
       httpFlush(http);
+      httpReconnect(http);
 
      /*
       * See if we can do local authentication...
@@ -1108,7 +1117,10 @@ cupsGetPPD(const char *name)		/* I - Printer name */
   cupsLangFree(language);
 
   if (!printer[0])
+  {
+    last_error = IPP_NOT_FOUND;
     return (NULL);
+  }
 
  /*
   * Reconnect to the correct server as needed...
@@ -1136,6 +1148,7 @@ cupsGetPPD(const char *name)		/* I - Printer name */
     * Can't open file; close the server connection and return NULL...
     */
 
+    last_error = IPP_INTERNAL_ERROR;
     httpFlush(cups_server);
     httpClose(cups_server);
     cups_server = NULL;
@@ -1278,6 +1291,22 @@ cupsGetPPD(const char *name)		/* I - Printer name */
 
   if (status != HTTP_OK)
   {
+    switch (status)
+    {
+      case HTTP_NOT_FOUND :
+          last_error = IPP_NOT_FOUND;
+	  break;
+      case HTTP_ERROR :
+          last_error = IPP_SERVICE_UNAVAILABLE;
+	  break;
+      case HTTP_UNAUTHORIZED :
+          last_error = IPP_NOT_AUTHORIZED;
+	  break;
+      default :
+          last_error = IPP_INTERNAL_ERROR;
+	  break;
+    }
+
     unlink(filename);
     httpFlush(cups_server);
     httpClose(cups_server);
@@ -1624,9 +1653,7 @@ cups_connect(const char *name,		/* I - Destination (printer[@host]) */
 	     char       *printer,	/* O - Printer name [HTTP_MAX_URI] */
              char       *hostname)	/* O - Hostname [HTTP_MAX_URI] */
 {
-  char		hostbuf[HTTP_MAX_URI];	/* Name of host */
-  static char	printerbuf[HTTP_MAX_URI];
-					/* Name of printer or class */
+  char	hostbuf[HTTP_MAX_URI];		/* Name of host */
 
 
   DEBUG_printf(("cups_connect(\"%s\", %p, %p)\n", name, printer, hostname));
@@ -1637,8 +1664,13 @@ cups_connect(const char *name,		/* I - Destination (printer[@host]) */
     return (NULL);
   }
 
-  if (sscanf(name, "%1023[^@]@%1023s", printerbuf, hostbuf) == 1)
-    strlcpy(hostbuf, cupsServer(), sizeof(hostbuf));
+ /*
+  * All jobs are now queued to cupsServer() to avoid hostname
+  * resolution problems and to ensure that the user sees all
+  * locally queued jobs locally.
+  */
+
+  strlcpy(hostbuf, cupsServer(), sizeof(hostbuf));
 
   if (hostname != NULL)
     strlcpy(hostname, hostbuf, HTTP_MAX_URI);
@@ -1646,9 +1678,9 @@ cups_connect(const char *name,		/* I - Destination (printer[@host]) */
     hostname = hostbuf;
 
   if (printer != NULL)
-    strlcpy(printer, printerbuf, HTTP_MAX_URI);
+    strlcpy(printer, name, HTTP_MAX_URI);
   else
-    printer = printerbuf;
+    printer = (char *)name;
 
   if (cups_server != NULL)
   {
@@ -1693,13 +1725,19 @@ cups_local_auth(http_t *http)	/* I - Connection */
   const char	*root;		/* Server root directory */
 
 
+  DEBUG_printf(("cups_local_auth(http=%p) hostaddr=%08x, hostname=\"%s\"\n",
+                http, ntohl(http->hostaddr.sin_addr.s_addr), http->hostname));
+
  /*
   * See if we are accessing localhost...
   */
 
   if (ntohl(http->hostaddr.sin_addr.s_addr) != 0x7f000001 &&
       strcasecmp(http->hostname, "localhost") != 0)
+  {
+    DEBUG_puts("cups_local_auth: Not a local connection!");
     return (0);
+  }
 
  /*
   * Try opening a certificate file for this PID.  If that fails,
@@ -1713,12 +1751,19 @@ cups_local_auth(http_t *http)	/* I - Connection */
   snprintf(filename, sizeof(filename), "%s/certs/%d", root, pid);
   if ((fp = fopen(filename, "r")) == NULL && pid > 0)
   {
+    DEBUG_printf(("cups_local_auth: Unable to open file %s: %s\n",
+                  filename, strerror(errno)));
+
     snprintf(filename, sizeof(filename), "%s/certs/0", root);
     fp = fopen(filename, "r");
   }
 
   if (fp == NULL)
+  {
+    DEBUG_printf(("cups_local_auth: Unable to open file %s: %s\n",
+                  filename, strerror(errno)));
     return (0);
+  }
 
  /*
   * Read the certificate from the file...
@@ -1733,11 +1778,14 @@ cups_local_auth(http_t *http)	/* I - Connection */
 
   snprintf(authstring, sizeof(authstring), "Local %s", certificate);
 
+  DEBUG_printf(("cups_local_auth: Returning authstring = \"%s\"\n",
+                authstring));
+
   return (1);
 #endif /* WIN32 || __EMX__ */
 }
 
 
 /*
- * End of "$Id: util.c,v 1.96 2002/05/31 13:02:19 mike Exp $".
+ * End of "$Id: util.c,v 1.103 2002/12/19 15:08:46 mike Exp $".
  */
