@@ -1,9 +1,9 @@
 /*
- * "$Id: pstops.c,v 1.83 2002/05/28 19:04:50 mike Exp $"
+ * "$Id: pstops.c,v 1.93 2002/12/17 18:59:28 swdev Exp $"
  *
  *   PostScript filter for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1993-2002 by Easy Software Products.
+ *   Copyright 1993-2003 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -60,8 +60,8 @@
 #define LAYOUT_RLBT	2	/* Right to left, bottom to top */
 #define LAYOUT_RLTB	3	/* Right to left, top to bottom */
 #define LAYOUT_BTLR	4	/* Bottom to top, left to right */
-#define LAYOUT_TBLR	5	/* Bottom to top, right to left */
-#define LAYOUT_BTRL	6	/* Top to bottom, left to right */
+#define LAYOUT_TBLR	5	/* Top to bottom, left to right */
+#define LAYOUT_BTRL	6	/* Bottom to top, right to left */
 #define LAYOUT_TBRL	7	/* Top to bottom, right to left */
 
 #define LAYOUT_NEGATEY	1
@@ -270,6 +270,10 @@ main(int  argc,			/* I - Number of command-line arguments */
   if ((val = cupsGetOption("brightness", num_options, options)) != NULL)
     b = atoi(val) * 0.01f;
 
+  if ((val = cupsGetOption("mirror", num_options, options)) != NULL &&
+      strcasecmp(val, "True") == 0)
+    Flip = 1;
+
  /*
   * See if we have to filter the fast or slow way...
   */
@@ -310,7 +314,13 @@ main(int  argc,			/* I - Number of command-line arguments */
   if (sloworder || slowcollate)
   {
     tempfd = cupsTempFd(tempfile, sizeof(tempfile));
-    temp   = fdopen(tempfd, "wb+");
+    if (tempfd < 0)
+    {
+      perror("ERROR: Unable to open temp file");
+      temp = NULL;
+    }
+    else
+      temp = fdopen(tempfd, "wb+");
 
     if (temp == NULL)
       slowcollate = sloworder = 0;
@@ -353,17 +363,37 @@ main(int  argc,			/* I - Number of command-line arguments */
   sent_setup  = 0;
 
   if (Copies != 1 && (!Collate || !slowcollate))
+  {
+   /*
+    * Tell the document processor the copy and duplex options
+    * that are required...
+    */
+
     printf("%%%%Requirements: numcopies(%d)%s%s\n", Copies,
            Collate ? " collate" : "",
 	   Duplex ? " duplex" : "");
-  else if (Duplex)
-    puts("%%%%Requirements: duplex\n");
 
- /*
-  * Apple uses RBI comments for various non-PPD options...
-  */
+   /*
+    * Apple uses RBI comments for various non-PPD options...
+    */
 
-  printf("%%RBINumCopies: %d\n", Copies);
+    printf("%%RBINumCopies: %d\n", Copies);
+  }
+  else
+  {
+   /*
+    * Tell the document processor the duplex option that is required...
+    */
+
+    if (Duplex)
+      puts("%%Requirements: duplex\n");
+
+   /*
+    * Apple uses RBI comments for various non-PPD options...
+    */
+
+    puts("%RBINumCopies: 1");
+  }
 
  /*
   * Figure out if we should use ESPshowpage or not...
@@ -381,10 +411,11 @@ main(int  argc,			/* I - Number of command-line arguments */
     UseESPsp = 1;
   }
 
-  if (strncmp(line, "%!PS-Adobe-", 11) == 0)
+  if (strncmp(line, "%!PS-Adobe-", 11) == 0 && strstr(line, "EPSF") == NULL)
   {
    /*
-    * OK, we have DSC comments; read until we find a %%Page comment...
+    * OK, we have DSC comments and this isn't an EPS file; read until we
+    * find a %%Page comment...
     */
 
     puts("%%Pages: (atend)");
@@ -418,23 +449,19 @@ main(int  argc,			/* I - Number of command-line arguments */
 	fputs(line, stdout);
         level --;
       }
-      else if (strncmp(line, "%%Orientation", 13) == 0 && level == 0)
+      else if (strncmp(line, "%cupsRotation:", 14) == 0 && level == 0)
       {
        /*
         * Reset orientation of document?
 	*/
 
-        if (strstr(line, "Landscape") != NULL && !(Orientation & 1))
+        int orient = (atoi(line + 14) / 90) & 3;
+
+        if (orient != Orientation)
 	{
-	  Orientation ++;
+	  Orientation = (4 - Orientation + orient) & 3;
 	  UpdatePageVars();
-	  Orientation = 1;
-	}
-        else if (strstr(line, "Portrait") != NULL && (Orientation & 1))
-	{
-	  Orientation = 4 - Orientation;
-	  UpdatePageVars();
-	  Orientation = 0;
+	  Orientation = orient;
 	}
       }
       else if (strncmp(line, "%%BeginProlog", 13) == 0 && level == 0)
@@ -865,6 +892,9 @@ main(int  argc,			/* I - Number of command-line arguments */
     puts("%%Trailer");
     printf("%%%%Pages: %d\n", page - 1);
 
+    if (UseESPsp)
+      puts("userdict/showpage/ESPshowpage load put\n");
+
     while (psgets(line, sizeof(line), fp) != NULL)
     {
       if (strcmp(line, "\004") != 0 &&
@@ -1646,5 +1676,5 @@ start_nup(int number,		/* I - Page number */
 
 
 /*
- * End of "$Id: pstops.c,v 1.83 2002/05/28 19:04:50 mike Exp $".
+ * End of "$Id: pstops.c,v 1.93 2002/12/17 18:59:28 swdev Exp $".
  */
