@@ -1,9 +1,9 @@
 /*
- * "$Id: classes.c,v 1.42 2002/05/16 13:44:58 mike Exp $"
+ * "$Id: classes.c,v 1.44 2002/12/17 19:00:14 swdev Exp $"
  *
  *   Printer class routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -64,12 +64,6 @@ AddClass(const char *name)	/* I - Name of class */
     c->type = CUPS_PRINTER_CLASS;
     snprintf(c->uri, sizeof(c->uri), "ipp://%s:%d/classes/%s", ServerName,
              ntohs(Listeners[0].address.sin_port), name);
-
-   /*
-    * Set the printer attributes to make this a class.
-    */
-
-    SetPrinterAttrs(c);
   }
 
   return (c);
@@ -350,7 +344,11 @@ LoadAllClasses(void)
 
   snprintf(line, sizeof(line), "%s/classes.conf", ServerRoot);
   if ((fp = fopen(line, "r")) == NULL)
+  {
+    LogMessage(L_ERROR, "LoadAllClasses: Unable to open %s - %s", line,
+               strerror(errno));
     return;
+  }
 
  /*
   * Read class configurations until we hit EOF...
@@ -414,6 +412,8 @@ LoadAllClasses(void)
       {
         line[len - 1] = '\0';
 
+        LogMessage(L_DEBUG, "LoadAllClasses: Loading class %s...", value);
+
         p = AddClass(value);
 	p->accepting = 1;
 	p->state     = IPP_PRINTER_IDLE;
@@ -691,5 +691,5 @@ SaveAllClasses(void)
 
 
 /*
- * End of "$Id: classes.c,v 1.42 2002/05/16 13:44:58 mike Exp $".
+ * End of "$Id: classes.c,v 1.44 2002/12/17 19:00:14 swdev Exp $".
  */
