@@ -1,9 +1,9 @@
 /*
- * "$Id: conf.c,v 1.104 2002/05/22 19:11:24 mike Exp $"
+ * "$Id: conf.c,v 1.111 2002/12/17 22:08:26 mike Exp $"
  *
  *   Configuration routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -91,7 +91,9 @@ static var_t	variables[] =
   { "DefaultLanguage",	DefaultLanguage,	VAR_STRING,	sizeof(DefaultLanguage) },
   { "DocumentRoot",	DocumentRoot,		VAR_STRING,	sizeof(DocumentRoot) },
   { "ErrorLog",		ErrorLog,		VAR_STRING,	sizeof(ErrorLog) },
+  { "FileDevice",	&FileDevice,		VAR_BOOLEAN,	0 },
   { "FilterLimit",	&FilterLimit,		VAR_INTEGER,	0 },
+  { "FilterNice",	&FilterNice,		VAR_INTEGER,	0 },
   { "FontPath",		FontPath,		VAR_STRING,	sizeof(FontPath) },
   { "HideImplicitMembers", &HideImplicitMembers, VAR_BOOLEAN,	0 },
   { "ImplicitClasses",	&ImplicitClasses,	VAR_BOOLEAN,	0 },
@@ -102,6 +104,8 @@ static var_t	variables[] =
   { "ListenBackLog",	&ListenBackLog,		VAR_INTEGER,	0 },
   { "LogFilePerm",	&LogFilePerm,		VAR_INTEGER,	0 },
   { "MaxClients",	&MaxClients,		VAR_INTEGER,	0 },
+  { "MaxClientsPerHost",&MaxClientsPerHost,	VAR_INTEGER,	0 },
+  { "MaxCopies",	&MaxCopies,		VAR_INTEGER,	0 },
   { "MaxJobs",		&MaxJobs,		VAR_INTEGER,	0 },
   { "MaxJobsPerPrinter",&MaxJobsPerPrinter,	VAR_INTEGER,	0 },
   { "MaxJobsPerUser",	&MaxJobsPerUser,	VAR_INTEGER,	0 },
@@ -116,6 +120,7 @@ static var_t	variables[] =
   { "RequestRoot",	RequestRoot,		VAR_STRING,	sizeof(RequestRoot) },
   { "RIPCache",		RIPCache,		VAR_STRING,	sizeof(RIPCache) },
   { "RunAsUser", 	&RunAsUser,		VAR_BOOLEAN,	0 },
+  { "RootCertDuration", &RootCertDuration,	VAR_INTEGER,	0 },
   { "ServerAdmin",	ServerAdmin,		VAR_STRING,	sizeof(ServerAdmin) },
   { "ServerBin",	ServerBin,		VAR_STRING,	sizeof(ServerBin) },
 #ifdef HAVE_LIBSSL
@@ -170,6 +175,7 @@ ReadConfiguration(void)
   * Free all memory...
   */
 
+  FreeAllJobs();
   DeleteAllClasses();
   DeleteAllLocations();
   DeleteAllPrinters();
@@ -306,8 +312,10 @@ ReadConfiguration(void)
   ConfigFilePerm      = 0600;
   LogFilePerm         = 0644;
 
+  FileDevice          = FALSE;
   FilterLevel         = 0;
   FilterLimit         = 0;
+  FilterNice          = 0;
   HostNameLookups     = FALSE;
   ImplicitClasses     = TRUE;
   ImplicitAnyClasses  = FALSE;
@@ -317,8 +325,10 @@ ReadConfiguration(void)
   ListenBackLog       = SOMAXCONN;
   LogLevel            = L_ERROR;
   MaxClients          = 100;
+  MaxClientsPerHost   = 0;
   MaxLogSize          = 1024 * 1024;
   MaxRequestSize      = 0;
+  RootCertDuration    = 300;
   RunAsUser           = FALSE;
   Timeout             = DEFAULT_TIMEOUT;
 
@@ -339,6 +349,7 @@ ReadConfiguration(void)
   MaxJobs             = 500;
   MaxJobsPerUser      = 0;
   MaxJobsPerPrinter   = 0;
+  MaxCopies           = 100;
 
  /*
   * Read the configuration file...
@@ -506,6 +517,24 @@ ReadConfiguration(void)
   if (Classification[0])
     LogMessage(L_INFO, "Security set to \"%s\"", Classification);
 
+ /*
+  * Update the MaxClientsPerHost value, as needed...
+  */
+
+  if (MaxClientsPerHost <= 0)
+  {
+    MaxClientsPerHost = MaxClients / 10;
+
+    if (MaxClientsPerHost < 4)
+      MaxClientsPerHost = 4;
+  }
+
+  if (MaxClientsPerHost > MaxClients)
+    MaxClientsPerHost = MaxClients;
+
+  LogMessage(L_INFO, "Allowing up to %d client connections per host.",
+             MaxClientsPerHost);
+
  /*
   * Read the MIME type and conversion database...
   */
@@ -571,6 +600,7 @@ ReadConfiguration(void)
   * Check for queued jobs...
   */
 
+  LoadAllJobs();
   CheckJobs();
 
   return (1);
@@ -1882,5 +1912,5 @@ get_address(char               *value,		/* I - Value string */
 
 
 /*
- * End of "$Id: conf.c,v 1.104 2002/05/22 19:11:24 mike Exp $".
+ * End of "$Id: conf.c,v 1.111 2002/12/17 22:08:26 mike Exp $".
  */
