@@ -1,10 +1,10 @@
 /*
- * "$Id: conf.h,v 1.44 2002/05/15 01:52:17 mike Exp $"
+ * "$Id: conf.h,v 1.50 2002/12/17 22:08:26 mike Exp $"
  *
  *   Configuration file definitions for the Common UNIX Printing System (CUPS)
  *   scheduler.
  *
- *   Copyright 1997-2002 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -110,6 +110,10 @@ VAR int			ClassifyOverride	VALUE(0),
 					/* Log level */
 			MaxClients		VALUE(0),
 					/* Maximum number of clients */
+			MaxClientsPerHost	VALUE(0),
+					/* Maximum number of clients per host */
+			MaxCopies		VALUE(100),
+					/* Maximum number of copies per job */
 			MaxLogSize		VALUE(1024 * 1024),
 					/* Maximum size of log files */
 			MaxRequestSize		VALUE(0),
@@ -128,10 +132,16 @@ VAR int			ClassifyOverride	VALUE(0),
 					/* Create AnyPrinter classes? */
 			HideImplicitMembers	VALUE(TRUE),
 					/* Hide implicit class members? */
+			FileDevice		VALUE(FALSE),
+					/* Allow file: devices? */
 			FilterLimit		VALUE(0),
 					/* Max filter cost at any time */
 			FilterLevel		VALUE(0),
 					/* Current filter level */
+			FilterNice		VALUE(0),
+					/* Nice value for filters */
+			RootCertDuration	VALUE(300),
+					/* Root certificate update interval */
 			RunAsUser		VALUE(FALSE),
 					/* Run as unpriviledged user? */
 			PrintcapFormat		VALUE(PRINTCAP_BSD);
@@ -173,5 +183,5 @@ extern int	LogPage(job_t *job, const char *page);
 
 
 /*
- * End of "$Id: conf.h,v 1.44 2002/05/15 01:52:17 mike Exp $".
+ * End of "$Id: conf.h,v 1.50 2002/12/17 22:08:26 mike Exp $".
  */
