@@ -1,9 +1,9 @@
 /*
- * "$Id: dirsvc.c,v 1.95 2002/05/16 13:45:00 mike Exp $"
+ * "$Id: dirsvc.c,v 1.102 2002/12/17 19:00:15 swdev Exp $"
  *
  *   Directory services routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -87,6 +87,31 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
 
   httpSeparate(uri, method, username, host, &port, resource);
 
+ /*
+  * Determine if the URI contains any illegal characters in it...
+  */
+
+  if (strncmp(uri, "ipp://", 6) != 0 ||
+      !host[0] ||
+      (strncmp(resource, "/printers/", 10) != 0 &&
+       strncmp(resource, "/classes/", 9) != 0))
+  {
+    LogMessage(L_ERROR, "ProcessBrowseData: Bad printer URI in browse data: %s",
+               uri);
+    return;
+  }
+
+  if (strchr(resource, '?') != NULL ||
+      (strncmp(resource, "/printers/", 10) == 0 &&
+       strchr(resource + 10, '/') != NULL) ||
+      (strncmp(resource, "/classes/", 9) == 0 &&
+       strchr(resource + 9, '/') != NULL))
+  {
+    LogMessage(L_ERROR, "ProcessBrowseData: Bad resource in browse data: %s",
+               resource);
+    return;
+  }
+    
  /*
   * OK, this isn't a local printer; see if we already have it listed in
   * the Printers list, and add it if not...
@@ -180,7 +205,8 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
       * Force the URI to point to the real server...
       */
 
-      p->type = type;
+      p->type      = type;
+      p->accepting = 1;
       strlcpy(p->uri, uri, sizeof(p->uri));
       strlcpy(p->device_uri, uri, sizeof(p->device_uri));
       strlcpy(p->hostname, host, sizeof(p->hostname));
@@ -254,7 +280,8 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
       * Force the URI to point to the real server...
       */
 
-      p->type = type;
+      p->type      = type;
+      p->accepting = 1;
       strlcpy(p->hostname, host, sizeof(p->hostname));
       strlcpy(p->uri, uri, sizeof(p->uri));
       strlcpy(p->device_uri, uri, sizeof(p->device_uri));
@@ -268,7 +295,6 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
   */
 
   p->state       = state;
-  p->accepting   = state != IPP_PRINTER_STOPPED;
   p->browse_time = time(NULL);
 
   if (p->type != type)
@@ -317,8 +343,12 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
   */
 
   if (DefaultPrinter == NULL && Printers != NULL)
+  {
     DefaultPrinter = Printers;
 
+    WritePrintcap();
+  }
+
  /*
   * Do auto-classing if needed...
   */
@@ -374,6 +404,9 @@ ProcessBrowseData(const char   *uri,	/* I - URI of printer/class */
 	  pclass->accepting = 1;
 	  pclass->state     = IPP_PRINTER_IDLE;
 
+          strcpy(pclass->location, p->location);
+          strcpy(pclass->info, p->info);
+
           SetPrinterAttrs(pclass);
 
           LogMessage(L_INFO, "Added implicit class \"%s\"...", name);
@@ -1116,12 +1149,12 @@ UpdateCUPSBrowse(void)
 
   if (auth == AUTH_DENY)
   {
-    LogMessage(L_DEBUG, "UpdateBrowseList: Refused %d bytes from %s", bytes,
+    LogMessage(L_DEBUG, "UpdateCUPSBrowse: Refused %d bytes from %s", bytes,
                srcname);
     return;
   }
 
-  LogMessage(L_DEBUG2, "UpdateBrowseList: (%d bytes from %s) %s", bytes, srcname,
+  LogMessage(L_DEBUG2, "UpdateCUPSBrowse: (%d bytes from %s) %s", bytes, srcname,
              packet);
 
  /*
@@ -1131,7 +1164,7 @@ UpdateCUPSBrowse(void)
   if (sscanf(packet, "%x%x%1023s", (unsigned *)&type, (unsigned *)&state,
              uri) < 3)
   {
-    LogMessage(L_WARN, "UpdateBrowseList: Garbled browse packet - %s",
+    LogMessage(L_WARN, "UpdateCUPSBrowse: Garbled browse packet - %s",
                packet);
     return;
   }
@@ -1226,7 +1259,7 @@ UpdateCUPSBrowse(void)
                  (struct sockaddr *)&(Relays[i].to),
 		 sizeof(struct sockaddr_in)) <= 0)
       {
-	LogMessage(L_ERROR, "UpdateBrowseList: sendto failed for relay %d - %s.",
+	LogMessage(L_ERROR, "UpdateCUPSBrowse: sendto failed for relay %d - %s.",
 	           i + 1, strerror(errno));
 	return;
       }
@@ -1766,6 +1799,12 @@ UpdateSLPBrowse(void)
 
   for (; s; s = next)
   {
+   /*
+    * Save the "next" pointer...
+    */
+
+    next = s->next;
+
    /* 
     * Load a printer_t structure with the SLP service attributes...
     */
@@ -1804,10 +1843,9 @@ UpdateSLPBrowse(void)
     }
 
    /*
-    * Save the "next" pointer and free this listing...
+    * Free this listing...
     */
 
-    next = s->next;
     free(s);
   }       
 
@@ -1817,5 +1855,5 @@ UpdateSLPBrowse(void)
 
 
 /*
- * End of "$Id: dirsvc.c,v 1.95 2002/05/16 13:45:00 mike Exp $".
+ * End of "$Id: dirsvc.c,v 1.102 2002/12/17 19:00:15 swdev Exp $".
  */
