@@ -1,9 +1,9 @@
 /*
- * "$Id: job.c,v 1.154 2002/05/16 13:45:01 mike Exp $"
+ * "$Id: job.c,v 1.180 2002/12/17 19:00:16 swdev Exp $"
  *
  *   Job management routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -29,6 +29,7 @@
  *   CheckJobs()          - Check the pending jobs and start any if the
  *                          destination is available.
  *   CleanJobs()          - Clean out old jobs.
+ *   FreeAllJobs()        - Free all jobs from memory.
  *   FindJob()            - Find the specified job.
  *   GetPrinterJobCount() - Get the number of pending, processing,
  *                          or held jobs in a printer or class.
@@ -71,7 +72,7 @@ static ipp_state_t	ipp_write_file(const char *filename, ipp_t *ipp);
 static void		set_time(job_t *job, const char *name);
 static int		start_process(const char *command, char *argv[],
 			              char *envp[], int in, int out, int err,
-				      int root);
+				      int root, int *pid);
 
 
 /*
@@ -354,6 +355,32 @@ CleanJobs(void)
 }
 
 
+/*
+ * 'FreeAllJobs()' - Free all jobs from memory.
+ */
+
+void
+FreeAllJobs(void)
+{
+  job_t	*job,		/* Current job */
+	*next;		/* Next job */
+
+
+  StopAllJobs();
+
+  for (job = Jobs; job; job = next)
+  {
+    next = job->next;
+
+    ippDelete(job->attrs);
+    free(job->filetypes);
+    free(job);
+  }
+
+  Jobs = NULL;
+}
+
+
 /*
  * 'FindJob()' - Find the specified job.
  */
@@ -476,8 +503,16 @@ LoadAllJobs(void)
   * First open the requests directory...
   */
 
+  LogMessage(L_DEBUG, "LoadAllJobs: Scanning %s...", RequestRoot);
+
+  NumJobs = 0;
+
   if ((dir = opendir(RequestRoot)) == NULL)
+  {
+    LogMessage(L_ERROR, "LoadAllJobs: Unable to open spool directory %s: %s",
+               RequestRoot, strerror(errno));
     return;
+  }
 
  /*
   * Read all the c##### files...
@@ -511,6 +546,9 @@ LoadAllJobs(void)
 
       job->id = atoi(dent->d_name + 1);
 
+      LogMessage(L_DEBUG, "LoadAllJobs: Loading attributes for job %d...\n",
+                 job->id);
+
       if (job->id >= NextJobId)
         NextJobId = job->id + 1;
 
@@ -602,19 +640,6 @@ LoadAllJobs(void)
       strlcpy(job->username, attr->values[0].string.text,
               sizeof(job->username));
 
-      if (job->state->values[0].integer == IPP_JOB_HELD)
-      {
-	if ((attr = ippFindAttribute(job->attrs, "job-hold-until", IPP_TAG_KEYWORD)) == NULL)
-          attr = ippFindAttribute(job->attrs, "job-hold-until", IPP_TAG_NAME);
-
-        if (attr == NULL)
-          job->state->values[0].integer = IPP_JOB_PENDING;
-	else
-          SetJobHoldUntil(job->id, attr->values[0].string.text);
-      }
-      else if (job->state->values[0].integer == IPP_JOB_PROCESSING)
-        job->state->values[0].integer = IPP_JOB_PENDING;
-
      /*
       * Insert the job into the array, sorting by job priority and ID...
       */
@@ -634,6 +659,23 @@ LoadAllJobs(void)
 	Jobs = job;
 
       NumJobs ++;
+
+     /*
+      * Set the job hold-until time and state...
+      */
+
+      if (job->state->values[0].integer == IPP_JOB_HELD)
+      {
+	if ((attr = ippFindAttribute(job->attrs, "job-hold-until", IPP_TAG_KEYWORD)) == NULL)
+          attr = ippFindAttribute(job->attrs, "job-hold-until", IPP_TAG_NAME);
+
+        if (attr == NULL)
+          job->state->values[0].integer = IPP_JOB_PENDING;
+	else
+          SetJobHoldUntil(job->id, attr->values[0].string.text);
+      }
+      else if (job->state->values[0].integer == IPP_JOB_PROCESSING)
+        job->state->values[0].integer = IPP_JOB_PENDING;
     }
 
  /*
@@ -652,6 +694,9 @@ LoadAllJobs(void)
       jobid  = atoi(dent->d_name + 1);
       fileid = atoi(dent->d_name + 7);
 
+      LogMessage(L_DEBUG, "LoadAllJobs: Auto-typing document file %s...",
+                 dent->d_name);
+
       snprintf(filename, sizeof(filename), "%s/%s", RequestRoot, dent->d_name);
 
       if ((job = FindJob(jobid)) == NULL)
@@ -729,7 +774,8 @@ MoveJob(int        id,		/* I - Job ID */
         break;
 
       strlcpy(current->dest, dest, sizeof(current->dest));
-      current->dtype = p->type & (CUPS_PRINTER_CLASS | CUPS_PRINTER_REMOTE);
+      current->dtype = p->type & (CUPS_PRINTER_CLASS | CUPS_PRINTER_REMOTE |
+                                  CUPS_PRINTER_IMPLICIT);
 
       if ((attr = ippFindAttribute(current->attrs, "job-printer-uri", IPP_TAG_URI)) != NULL)
       {
@@ -1049,7 +1095,6 @@ StartJob(int       id,		/* I - Job ID */
 		title[IPP_MAX_NAME],
 				/* Job title string */
 		copies[255],	/* # copies string */
-		options[16384],	/* Full list of options */
 		*envp[20],	/* Environment variables */
 		path[1024],	/* PATH environment variable */
 		language[255],	/* LANG environment variable */
@@ -1061,6 +1106,8 @@ StartJob(int       id,		/* I - Job ID */
 		device_uri[1024],
 				/* DEVICE_URI environment variable */
 		ppd[1024],	/* PPD environment variable */
+		class_name[255],
+				/* CLASS environment variable */
 		printer_name[255],
 				/* PRINTER environment variable */
 		root[1024],	/* CUPS_SERVERROOT environment variable */
@@ -1070,6 +1117,8 @@ StartJob(int       id,		/* I - Job ID */
 		nlspath[1024],	/* NLSPATH environment variable */
 		datadir[1024],	/* CUPS_DATADIR environment variable */
 		fontpath[1050];	/* CUPS_FONTPATH environment variable */
+  static char	*options = NULL;/* Full list of options */
+  static int	optlength = 0;	/* Length of option buffer */
 
 
   LogMessage(L_DEBUG, "StartJob(%d, %p)", id, printer);
@@ -1099,12 +1148,15 @@ StartJob(int       id,		/* I - Job ID */
   num_filters   = 0;
   current->cost = 0;
 
-  if (printer->type & CUPS_PRINTER_REMOTE)
+  if (printer->raw)
   {
    /*
-    * Remote jobs go directly to the remote job...
+    * Remote jobs and raw queues go directly to the printer without
+    * filtering...
     */
 
+    LogMessage(L_DEBUG, "StartJob: Sending job to queue tagged as raw...");
+
     filters = NULL;
   }
   else
@@ -1114,12 +1166,17 @@ StartJob(int       id,		/* I - Job ID */
     */
 
     filters = mimeFilter(MimeDatabase, current->filetypes[current->current_file],
-                         printer->filetype, &num_filters);
+                         printer->filetype, &num_filters, MAX_FILTERS - 1);
 
     if (num_filters == 0)
     {
       LogMessage(L_ERROR, "Unable to convert file %d to printable format for job %d!",
 	         current->current_file, current->id);
+      LogMessage(L_INFO, "Hint: Do you have ESP Ghostscript installed?");
+
+      if (LogLevel < L_DEBUG)
+        LogMessage(L_INFO, "Hint: Try setting the LogLevel to \"debug\".");
+
       current->current_file ++;
 
       if (current->current_file == current->num_files)
@@ -1236,6 +1293,45 @@ StartJob(int       id,		/* I - Job ID */
   * Building the options string is harder than it needs to be, but
   * for the moment we need to pass strings for command-line args and
   * not IPP attribute pointers... :)
+  *
+  * First allocate/reallocate the option buffer as needed...  Using
+  * twice the raw length of the IPP attributes provides enough space for
+  * all options that are encoded as text, as we only need 1 byte of
+  * overhead for normal attributes and 6 bytes for boolean attributes,
+  * and each attribute has at least 5 bytes of overhead in the IPP
+  * message...
+  */
+
+  i = 2 * ippLength(current->attrs);
+
+  if (i > optlength)
+  {
+    if (optlength == 0)
+      optptr = malloc(i);
+    else
+      optptr = realloc(options, i);
+
+    if (optptr == NULL)
+    {
+      LogMessage(L_CRIT, "StartJob: Unable to allocate %d bytes for option buffer for job %d!",
+                 i, id);
+
+      if (filters != NULL)
+        free(filters);
+
+      FilterLevel -= current->cost;
+      
+      CancelJob(id, 0);
+      return;
+    }
+
+    options   = optptr;
+    optlength = i;
+  }
+
+ /*
+  * Now loop through the attributes and convert them to the textual
+  * representation used by the filters...
   */
 
   optptr  = options;
@@ -1287,7 +1383,9 @@ StartJob(int       id,		/* I - Job ID */
 	  strcmp(attr->name, "job-priority") != 0)
 	continue;
 
-      if (strcmp(attr->name, "page-label") == 0 &&
+      if ((strcmp(attr->name, "page-label") == 0 ||
+           strcmp(attr->name, "page-border") == 0 ||
+           strncmp(attr->name, "number-up", 9) == 0) &&
 	  banner_page)
         continue;
 
@@ -1296,18 +1394,18 @@ StartJob(int       id,		/* I - Job ID */
       */
 
       if (optptr > options)
-	strlcat(optptr, " ", sizeof(options) - (optptr - options));
+	strlcat(optptr, " ", optlength - (optptr - options));
 
       if (attr->value_tag != IPP_TAG_BOOLEAN)
       {
-	strlcat(optptr, attr->name, sizeof(options) - (optptr - options));
-	strlcat(optptr, "=", sizeof(options) - (optptr - options));
+	strlcat(optptr, attr->name, optlength - (optptr - options));
+	strlcat(optptr, "=", optlength - (optptr - options));
       }
 
       for (i = 0; i < attr->num_values; i ++)
       {
 	if (i)
-	  strlcat(optptr, ",", sizeof(options) - (optptr - options));
+	  strlcat(optptr, ",", optlength - (optptr - options));
 
 	optptr += strlen(optptr);
 
@@ -1315,31 +1413,31 @@ StartJob(int       id,		/* I - Job ID */
 	{
 	  case IPP_TAG_INTEGER :
 	  case IPP_TAG_ENUM :
-	      snprintf(optptr, sizeof(options) - (optptr - options),
+	      snprintf(optptr, optlength - (optptr - options),
 	               "%d", attr->values[i].integer);
 	      break;
 
 	  case IPP_TAG_BOOLEAN :
 	      if (!attr->values[i].boolean)
-		strlcat(optptr, "no", sizeof(options) - (optptr - options));
+		strlcat(optptr, "no", optlength - (optptr - options));
 
 	  case IPP_TAG_NOVALUE :
 	      strlcat(optptr, attr->name,
-	              sizeof(options) - (optptr - options));
+	              optlength - (optptr - options));
 	      break;
 
 	  case IPP_TAG_RANGE :
 	      if (attr->values[i].range.lower == attr->values[i].range.upper)
-		snprintf(optptr, sizeof(options) - (optptr - options) - 1,
+		snprintf(optptr, optlength - (optptr - options) - 1,
 	        	 "%d", attr->values[i].range.lower);
               else
-		snprintf(optptr, sizeof(options) - (optptr - options) - 1,
+		snprintf(optptr, optlength - (optptr - options) - 1,
 	        	 "%d-%d", attr->values[i].range.lower,
 			 attr->values[i].range.upper);
 	      break;
 
 	  case IPP_TAG_RESOLUTION :
-	      snprintf(optptr, sizeof(options) - (optptr - options) - 1,
+	      snprintf(optptr, optlength - (optptr - options) - 1,
 	               "%dx%d%s", attr->values[i].resolution.xres,
 		       attr->values[i].resolution.yres,
 		       attr->values[i].resolution.units == IPP_RES_PER_INCH ?
@@ -1356,14 +1454,14 @@ StartJob(int       id,		/* I - Job ID */
 		  strchr(attr->values[i].string.text, '\t') != NULL ||
 		  strchr(attr->values[i].string.text, '\n') != NULL)
 	      {
-		strlcat(optptr, "\'", sizeof(options) - (optptr - options));
+		strlcat(optptr, "\'", optlength - (optptr - options));
 		strlcat(optptr, attr->values[i].string.text,
-		        sizeof(options) - (optptr - options));
-		strlcat(optptr, "\'", sizeof(options) - (optptr - options));
+		        optlength - (optptr - options));
+		strlcat(optptr, "\'", optlength - (optptr - options));
 	      }
 	      else
 		strlcat(optptr, attr->values[i].string.text,
-		        sizeof(options) - (optptr - options));
+		        optlength - (optptr - options));
 	      break;
 
           default :
@@ -1473,6 +1571,11 @@ StartJob(int       id,		/* I - Job ID */
   snprintf(datadir, sizeof(datadir), "CUPS_DATADIR=%s", DataDir);
   snprintf(fontpath, sizeof(fontpath), "CUPS_FONTPATH=%s", FontPath);
 
+  if (current->dtype & (CUPS_PRINTER_CLASS | CUPS_PRINTER_IMPLICIT))
+    snprintf(class_name, sizeof(class_name), "CLASS=%s", current->dest);
+  else
+    class_name[0] = '\0';
+
   if (Classification[0] && !banner_page)
   {
     if ((attr = ippFindAttribute(current->attrs, "job-sheets",
@@ -1496,6 +1599,9 @@ StartJob(int       id,		/* I - Job ID */
   else if (getenv("DYLD_LIBRARY_PATH") != NULL)
     snprintf(ldpath, sizeof(ldpath), "DYLD_LIBRARY_PATH=%s",
              getenv("DYLD_LIBRARY_PATH"));
+  else if (getenv("SHLIB_PATH") != NULL)
+    snprintf(ldpath, sizeof(ldpath), "SHLIB_PATH=%s",
+             getenv("SHLIB_PATH"));
   else
     ldpath[0] = '\0';
 
@@ -1522,7 +1628,8 @@ StartJob(int       id,		/* I - Job ID */
   envp[15] = ldpath;
   envp[16] = nlspath;
   envp[17] = classification;
-  envp[18] = NULL;
+  envp[18] = class_name;
+  envp[19] = NULL;
 
   LogMessage(L_DEBUG, "StartJob: envp = \"%s\",\"%s\",\"%s\",\"%s\","
                       "\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\",\"%s\","
@@ -1606,7 +1713,8 @@ StartJob(int       id,		/* I - Job ID */
                slot, filterfds[slot][0], filterfds[slot][1]);
 
     pid = start_process(command, argv, envp, filterfds[!slot][0],
-                        filterfds[slot][1], statusfds[1], 0);
+                        filterfds[slot][1], statusfds[1], 0,
+			current->procs + i);
 
     close(filterfds[!slot][0]);
     close(filterfds[!slot][1]);
@@ -1621,8 +1729,6 @@ StartJob(int       id,		/* I - Job ID */
       return;
     }
 
-    current->procs[i] = pid;
-
     LogMessage(L_INFO, "Started filter %s (PID %d) for job %d.",
                command, pid, current->id);
 
@@ -1652,7 +1758,8 @@ StartJob(int       id,		/* I - Job ID */
                slot, filterfds[slot][0], filterfds[slot][1]);
 
     pid = start_process(command, argv, envp, filterfds[!slot][0],
-			filterfds[slot][1], statusfds[1], 1);
+			filterfds[slot][1], statusfds[1], 1,
+			current->procs + i);
 
     close(filterfds[!slot][0]);
     close(filterfds[!slot][1]);
@@ -1667,8 +1774,6 @@ StartJob(int       id,		/* I - Job ID */
     }
     else
     {
-      current->procs[i] = pid;
-
       LogMessage(L_INFO, "Started backend %s (PID %d) for job %d.", command, pid,
                  current->id);
     }
@@ -1739,7 +1844,8 @@ StopJob(int id,			/* I - Job ID */
 
         FilterLevel -= current->cost;
 
-        if (current->status < 0)
+        if (current->status < 0 &&
+	    (current->dtype & (CUPS_PRINTER_CLASS | CUPS_PRINTER_IMPLICIT)) != 0)
 	  SetPrinterState(current->printer, IPP_PRINTER_STOPPED);
 	else if (current->printer->state != IPP_PRINTER_STOPPED)
 	  SetPrinterState(current->printer, IPP_PRINTER_IDLE);
@@ -1810,14 +1916,17 @@ UpdateJob(job_t *job)		/* I - Job to check */
   {
     job->bufused += bytes;
     job->buffer[job->bufused] = '\0';
-    lineptr = strchr(job->buffer, '\n');
+
+    if ((lineptr = strchr(job->buffer, '\n')) == NULL &&
+        job->bufused == (JOB_BUFFER_SIZE - 1))
+      lineptr  = job->buffer + job->bufused;
   }
   else if (bytes < 0 && errno == EINTR)
     return;
   else
   {
-    lineptr    = job->buffer + job->bufused;
-    lineptr[1] = 0;
+    lineptr  = job->buffer + job->bufused;
+    *lineptr = '\0';
   }
 
   if (job->bufused == 0 && bytes == 0)
@@ -1935,8 +2044,8 @@ UpdateJob(job_t *job)		/* I - Job to check */
       * Other status message; send it to the error_log file...
       */
 
-      if (loglevel != L_INFO)
-	LogMessage(loglevel, "%s", message);
+      if (loglevel != L_INFO || LogLevel == L_DEBUG2)
+	LogMessage(loglevel, "[Job %d] %s", job->id, message);
 
       if ((loglevel == L_INFO && !job->status) ||
 	  loglevel < L_INFO)
@@ -1985,6 +2094,13 @@ UpdateJob(job_t *job)		/* I - Job to check */
       StopJob(job->id, 0);
       job->state->values[0].integer = IPP_JOB_PENDING;
       SaveJob(job->id);
+
+     /*
+      * If the job was queued to a class, try requeuing it...
+      */
+
+      if (job->dtype & (CUPS_PRINTER_CLASS | CUPS_PRINTER_IMPLICIT))
+        CheckJobs();
     }
     else if (job->status > 0)
     {
@@ -2420,7 +2536,7 @@ ipp_write_file(const char *filename,	/* I - File to write to */
   int			fd;		/* File descriptor */
   int			i;		/* Looping var */
   int			n;		/* Length of data */
-  unsigned char		buffer[8192],	/* Data buffer */
+  unsigned char		buffer[32768],	/* Data buffer */
 			*bufptr;	/* Pointer into buffer */
   ipp_attribute_t	*attr;		/* Current attribute */
 
@@ -2626,9 +2742,12 @@ ipp_write_file(const char *filename,	/* I - File to write to */
 		    *bufptr++ = 0;
 		  }
 
-                  n = strlen(attr->values[i].string.text);
+                  if (attr->values[i].string.text != NULL)
+		    n = strlen(attr->values[i].string.text);
+		  else
+		    n = 0;
 
-                  if (n > sizeof(buffer))
+                  if (n > (sizeof(buffer) - 2))
 		    return (IPP_ERROR);
 
                   DEBUG_printf(("ipp_write_file: writing string = %d, \'%s\'\n", n,
@@ -2648,8 +2767,12 @@ ipp_write_file(const char *filename,	/* I - File to write to */
 
 	          *bufptr++ = n >> 8;
 		  *bufptr++ = n;
-		  memcpy(bufptr, attr->values[i].string.text, n);
-		  bufptr += n;
+
+		  if (n > 0)
+		  {
+		    memcpy(bufptr, attr->values[i].string.text, n);
+		    bufptr += n;
+		  }
 		}
 		break;
 
@@ -2792,11 +2915,15 @@ ipp_write_file(const char *filename,	/* I - File to write to */
 		    *bufptr++ = 0;
 		  }
 
-                  n = strlen(attr->values[i].string.charset) +
-		      strlen(attr->values[i].string.text) +
-		      4;
+                  n = 4;
 
-                  if (n > sizeof(buffer))
+		  if (attr->values[i].string.charset != NULL)
+                    n += strlen(attr->values[i].string.charset);
+
+		  if (attr->values[i].string.text != NULL)
+                    n += strlen(attr->values[i].string.text);
+
+                  if (n > (sizeof(buffer) - 2))
 		    return (IPP_ERROR);
 
                   if ((sizeof(buffer) - (bufptr - buffer)) < (n + 2))
@@ -2815,22 +2942,36 @@ ipp_write_file(const char *filename,	/* I - File to write to */
 		  *bufptr++ = n;
 
                  /* Length of charset */
-                  n = strlen(attr->values[i].string.charset);
+                  if (attr->values[i].string.charset != NULL)
+		    n = strlen(attr->values[i].string.charset);
+		  else
+		    n = 0;
+
 	          *bufptr++ = n >> 8;
 		  *bufptr++ = n;
 
                  /* Charset */
-		  memcpy(bufptr, attr->values[i].string.charset, n);
-		  bufptr += n;
+		  if (n > 0)
+		  {
+		    memcpy(bufptr, attr->values[i].string.charset, n);
+		    bufptr += n;
+		  }
 
                  /* Length of text */
-                  n = strlen(attr->values[i].string.text);
+                  if (attr->values[i].string.text != NULL)
+		    n = strlen(attr->values[i].string.text);
+		  else
+		    n = 0;
+
 	          *bufptr++ = n >> 8;
 		  *bufptr++ = n;
 
                  /* Text */
-		  memcpy(bufptr, attr->values[i].string.text, n);
-		  bufptr += n;
+		  if (n > 0)
+		  {
+		    memcpy(bufptr, attr->values[i].string.text, n);
+		    bufptr += n;
+		  }
 		}
 		break;
 
@@ -2862,7 +3003,7 @@ ipp_write_file(const char *filename,	/* I - File to write to */
 
                   n = attr->values[i].unknown.length;
 
-                  if (n > sizeof(buffer))
+                  if (n > (sizeof(buffer) - 2))
 		    return (IPP_ERROR);
 
                   if ((sizeof(buffer) - (bufptr - buffer)) < (n + 2))
@@ -2966,16 +3107,35 @@ start_process(const char *command,	/* I - Full path to command */
               int        infd,		/* I - Standard input file descriptor */
 	      int        outfd,		/* I - Standard output file descriptor */
 	      int        errfd,		/* I - Standard error file descriptor */
-	      int        root)		/* I - Run as root? */
+	      int        root,		/* I - Run as root? */
+              int        *pid)		/* O - Process ID */
 {
   int	fd;				/* Looping var */
-  int	pid;				/* Process ID */
+#if defined(HAVE_SIGACTION) && !defined(HAVE_SIGSET)
+  sigset_t		oldmask,	/* POSIX signal masks */
+			newmask;
+  struct sigaction	action;		/* POSIX signal handler */
+#endif /* HAVE_SIGACTION && !HAVE_SIGSET */
 
 
   LogMessage(L_DEBUG, "start_process(\"%s\", %p, %p, %d, %d, %d)",
              command, argv, envp, infd, outfd, errfd);
 
-  if ((pid = fork()) == 0)
+ /*
+  * Block signals before forking...
+  */
+
+#ifdef HAVE_SIGSET
+  sighold(SIGTERM);
+  sighold(SIGCHLD);
+#elif defined(HAVE_SIGACTION)
+  sigemptyset(&newmask);
+  sigaddset(&newmask, SIGTERM);
+  sigaddset(&newmask, SIGCHLD);
+  sigprocmask(SIG_BLOCK, &newmask, &oldmask);
+#endif /* HAVE_SIGSET */
+
+  if ((*pid = fork()) == 0)
   {
    /*
     * Child process goes here...
@@ -3000,6 +3160,14 @@ start_process(const char *command,	/* I - Full path to command */
     for (fd = 3; fd < MaxFDs; fd ++)
       close(fd);
 
+   /*
+    * Change the priority of the process based on the FilterNice setting.
+    * (this is not done for backends...)
+    */
+
+    if (!root)
+      nice(FilterNice);
+
    /*
     * Change user to something "safe"...
     */
@@ -3029,6 +3197,31 @@ start_process(const char *command,	/* I - Full path to command */
 
     umask(077);
 
+   /*
+    * Unblock signals before doing the exec...
+    */
+
+#ifdef HAVE_SIGSET
+    sigset(SIGTERM, SIG_DFL);
+    sigset(SIGCHLD, SIG_DFL);
+
+    sigrelse(SIGTERM);
+    sigrelse(SIGCHLD);
+#elif defined(HAVE_SIGACTION)
+    memset(&action, 0, sizeof(action));
+
+    sigemptyset(&action.sa_mask);
+    action.sa_handler = SIG_DFL;
+
+    sigaction(SIGTERM, &action, NULL);
+    sigaction(SIGCHLD, &action, NULL);
+
+    sigprocmask(SIG_SETMASK, &oldmask, NULL);
+#else
+    signal(SIGTERM, SIG_DFL);
+    signal(SIGCHLD, SIG_DFL);
+#endif /* HAVE_SIGSET */
+
    /*
     * Execute the command; if for some reason this doesn't work,
     * return the error code...
@@ -3040,7 +3233,7 @@ start_process(const char *command,	/* I - Full path to command */
 
     exit(errno);
   }
-  else if (pid < 0)
+  else if (*pid < 0)
   {
    /*
     * Error - couldn't fork a new process!
@@ -3048,13 +3241,20 @@ start_process(const char *command,	/* I - Full path to command */
 
     LogMessage(L_ERROR, "Unable to fork %s - %s.", command, strerror(errno));
 
-    return (0);
+    *pid = 0;
   }
 
-  return (pid);
+#ifdef HAVE_SIGSET
+  sigrelse(SIGTERM);
+  sigrelse(SIGCHLD);
+#elif defined(HAVE_SIGACTION)
+  sigprocmask(SIG_SETMASK, &oldmask, NULL);
+#endif /* HAVE_SIGSET */
+
+  return (*pid);
 }
 
 
 /*
- * End of "$Id: job.c,v 1.154 2002/05/16 13:45:01 mike Exp $".
+ * End of "$Id: job.c,v 1.180 2002/12/17 19:00:16 swdev Exp $".
  */
