@@ -1,9 +1,9 @@
 /*
- * "$Id: main.c,v 1.72 2002/05/16 13:45:02 mike Exp $"
+ * "$Id: main.c,v 1.85 2002/12/17 19:00:17 swdev Exp $"
  *
  *   Scheduler main loop for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -40,6 +40,7 @@
 #include "cupsd.h"
 #include <sys/resource.h>
 #include <syslog.h>
+#include <grp.h>
 
 #if defined(HAVE_MALLOC_H) && defined(HAVE_MALLINFO)
 #  include <malloc.h>
@@ -57,6 +58,7 @@
 static void	sigchld_handler(int sig);
 static void	sighup_handler(int sig);
 static void	sigterm_handler(int sig);
+static void	sigusr1_handler(int sig);
 static void	usage(void);
 
 
@@ -132,6 +134,10 @@ main(int  argc,			/* I - Number of command-line arguments */
 	      fg = 1;
 	      break;
 
+          case 'F' : /* Run in foreground, but still disconnect from terminal... */
+	      fg = -1;
+	      break;
+
 	  default : /* Unknown option */
               fprintf(stderr, "cupsd: Unknown option \'%c\' - aborting!\n", *opt);
 	      usage();
@@ -150,8 +156,50 @@ main(int  argc,			/* I - Number of command-line arguments */
   if (!fg)
   {
     if (fork() > 0)
-      return (0);
+    {
+     /*
+      * OK, wait for the child to startup and send us SIGUSR1...  We
+      * also need to ignore SIGHUP which might be sent by the init
+      * script to restart the scheduler...
+      */
 
+#ifdef HAVE_SIGSET /* Use System V signals over POSIX to avoid bugs */
+      sigset(SIGUSR1, sigusr1_handler);
+
+      sigset(SIGHUP, SIG_IGN);
+#elif defined(HAVE_SIGACTION)
+      memset(&action, 0, sizeof(action));
+      sigemptyset(&action.sa_mask);
+      sigaddset(&action.sa_mask, SIGUSR1);
+      action.sa_handler = sigusr1_handler;
+      sigaction(SIGUSR1, &action, NULL);
+
+      sigemptyset(&action.sa_mask);
+      action.sa_handler = SIG_IGN;
+      sigaction(SIGHUP, &action, NULL);
+#else
+      signal(SIGUSR1, sigusr1_handler);
+
+      signal(SIGHUP, SIG_IGN);
+#endif /* HAVE_SIGSET */
+
+      if (wait(&i) < 0)
+        i = 0;
+
+      if (i == 0)
+        return (0);
+
+      if (i >= 256)
+        fprintf(stderr, "cupsd: Child exited with status %d!\n", i / 256);
+      else
+        fprintf(stderr, "cupsd: Child exited on signal %d!\n", i);
+
+      return (i);
+    }
+  }
+
+  if (fg < 1)
+  {
    /*
     * Make sure we aren't tying up any filesystems...
     */
@@ -185,9 +233,15 @@ main(int  argc,			/* I - Number of command-line arguments */
 
   if (getenv("TZ") != NULL)
     snprintf(TZ, sizeof(TZ), "TZ=%s", getenv("TZ"));
+  else
+    TZ[0] = '\0';
 
   tzset();
 
+#ifdef LC_TIME
+  setlocale(LC_TIME, "");
+#endif /* LC_TIME */
+
  /*
   * Set the maximum number of files...
   */
@@ -231,6 +285,7 @@ main(int  argc,			/* I - Number of command-line arguments */
 
   sigemptyset(&action.sa_mask);
   sigaddset(&action.sa_mask, SIGTERM);
+  sigaddset(&action.sa_mask, SIGCHLD);
   action.sa_handler = sigterm_handler;
   sigaction(SIGTERM, &action, NULL);
 #else
@@ -283,10 +338,24 @@ main(int  argc,			/* I - Number of command-line arguments */
   InitCerts();
 
  /*
-  * Load all the jobs...
+  * If we are running in the background, signal the parent process that
+  * we are up and running...
   */
 
-  LoadAllJobs();
+  if (!fg)
+    kill(getppid(), SIGUSR1);
+
+ /*
+  * If the administrator has configured the server to run as an unpriviledged
+  * user, change to that user now...
+  */
+
+  if (RunAsUser)
+  {
+    setgid(Group);
+    setgroups(0, NULL);
+    setuid(User);
+  }
 
  /*
   * Loop forever...
@@ -338,15 +407,25 @@ main(int  argc,			/* I - Number of command-line arguments */
     input  = InputSet;
     output = OutputSet;
 
-    timeout.tv_sec  = 1;
-    timeout.tv_usec = 0;
-
     for (i = NumClients, con = Clients; i > 0; i --, con ++)
       if (con->http.used > 0)
-      {
-        timeout.tv_sec  = 0;
-	break;
-      }
+        break;
+
+    if (i)
+    {
+      timeout.tv_sec  = 0;
+      timeout.tv_usec = 0;
+    }
+    else
+    {
+     /*
+      * If we have no pending data from a client, see when we really
+      * need to wake up...
+      */
+
+      timeout.tv_sec  = 1;
+      timeout.tv_usec = 0;
+    }
 
     if ((i = select(MaxFDs, &input, &output, NULL, &timeout)) < 0)
     {
@@ -530,7 +609,7 @@ main(int  argc,			/* I - Number of command-line arguments */
     * Update the root certificate once every 5 minutes...
     */
 
-    if ((time(NULL) - RootCertTime) >= 300)
+    if ((time(NULL) - RootCertTime) >= RootCertDuration && RootCertDuration)
     {
      /*
       * Update the root certificate...
@@ -572,6 +651,7 @@ CatchChildSignals(void)
   memset(&action, 0, sizeof(action));
 
   sigemptyset(&action.sa_mask);
+  sigaddset(&action.sa_mask, SIGTERM);
   sigaddset(&action.sa_mask, SIGCHLD);
   action.sa_handler = sigchld_handler;
   sigaction(SIGCHLD, &action, NULL);
@@ -614,6 +694,7 @@ IgnoreChildSignals(void)
 static void
 sigchld_handler(int sig)	/* I - Signal number */
 {
+  int		olderrno;	/* Old errno value */
   int		status;		/* Exit status of child */
   int		pid;		/* Process ID of child */
   job_t		*job;		/* Current job */
@@ -622,6 +703,12 @@ sigchld_handler(int sig)	/* I - Signal number */
 
   (void)sig;
 
+ /*
+  * Save the original error value (wait might overwrite it...)
+  */
+
+  olderrno = errno;
+
 #ifdef HAVE_WAITPID
   while ((pid = waitpid(-1, &status, WNOHANG)) > 0)
 #elif defined(HAVE_WAIT3)
@@ -630,14 +717,7 @@ sigchld_handler(int sig)	/* I - Signal number */
   if ((pid = wait(&status)) > 0)
 #endif /* HAVE_WAITPID */
   {
-    DEBUG_printf(("sigcld_handler: pid = %d, status = %d\n", pid, status));
-
-   /*
-    * Delete certificates for CGI processes...
-    */
-
-    if (pid)
-      DeleteCert(pid);
+    DEBUG_printf(("sigchld_handler: pid = %d, status = %d\n", pid, status));
 
    /*
     * Ignore SIGTERM errors - that comes when a job is cancelled...
@@ -653,10 +733,27 @@ sigchld_handler(int sig)	/* I - Signal number */
       else
 	LogMessage(L_ERROR, "PID %d stopped with status %d!", pid,
 	           status / 256);
+
+      if (LogLevel < L_DEBUG)
+        LogMessage(L_INFO, "Hint: Try setting the LogLevel to \"debug\" to find out more.");
     }
+    else
+      LogMessage(L_DEBUG2, "PID %d exited with no errors.", pid);
+
+   /*
+    * Delete certificates for CGI processes...
+    */
+
+    if (pid)
+      DeleteCert(pid);
+
+   /*
+    * Lookup the PID in the jobs list...
+    */
 
     for (job = Jobs; job != NULL; job = job->next)
-      if (job->state->values[0].integer == IPP_JOB_PROCESSING)
+      if (job->state != NULL &&
+          job->state->values[0].integer == IPP_JOB_PROCESSING)
       {
 	for (i = 0; job->procs[i]; i ++)
           if (job->procs[i] == pid)
@@ -690,6 +787,12 @@ sigchld_handler(int sig)	/* I - Signal number */
       }
   }
 
+ /*
+  * Restore the original error value...
+  */
+
+  errno = olderrno;
+
 #ifdef HAVE_SIGSET
   sigset(SIGCHLD, sigchld_handler);
 #elif !defined(HAVE_SIGACTION)
@@ -722,7 +825,7 @@ sighup_handler(int sig)	/* I - Signal number */
  */
 
 static void
-sigterm_handler(int sig)
+sigterm_handler(int sig)		/* I - Signal */
 {
 #ifdef __sgi
   struct stat	statbuf;		/* Needed for checking lpsched FIFO */
@@ -749,7 +852,7 @@ sigterm_handler(int sig)
   if (Clients != NULL)
     free(Clients);
 
-  StopAllJobs();
+  FreeAllJobs();
 
   if (AccessFile != NULL)
     fclose(AccessFile);
@@ -791,6 +894,17 @@ sigterm_handler(int sig)
 }
 
 
+/*
+ * 'sigusr1_handler()' - Catch USR1 signals...
+ */
+
+static void
+sigusr1_handler(int sig)		/* I - Signal */
+{
+  (void)sig;	/* remove compiler warnings... */
+}
+
+
 /*
  * 'usage()' - Show scheduler usage.
  */
@@ -804,5 +918,5 @@ usage(void)
 
 
 /*
- * End of "$Id: main.c,v 1.72 2002/05/16 13:45:02 mike Exp $".
+ * End of "$Id: main.c,v 1.85 2002/12/17 19:00:17 swdev Exp $".
  */
