@@ -1,9 +1,9 @@
 /*
- * "$Id: ppds.c,v 1.23 2002/05/16 13:45:03 mike Exp $"
+ * "$Id: ppds.c,v 1.27 2002/12/17 19:00:17 swdev Exp $"
  *
  *   PPD scanning routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products.
+ *   Copyright 1997-2003 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -476,18 +476,21 @@ load_ppds(const char *d,		/* I - Actual directory */
   }		languages[] =
   {
     { "chinese",	"cn" },
+    { "danish",		"da" },
+    { "dutch",		"nl" },
     { "english",	"en" },
+    { "finnish",	"fi" },
     { "french",		"fr" },
     { "german",		"de" },
-    { "danish",		"da" },
-    { "finnish",	"fi" },
+    { "greek",		"el" },
     { "italian",	"it" },
-    { "dutch",		"du" },
     { "japanese",	"jp" },
     { "norwegian",	"no" },
     { "polish",		"pl" },
-    { "portugese",	"pt" },
+    { "portuguese",	"pt" },
     { "russian",	"ru" },
+    { "slovak",		"sk" },
+    { "spanish",	"es" },
     { "swedish",	"sv" },
     { "turkish",	"tr" }
   };
@@ -890,5 +893,5 @@ ppd_gets(buf_t *fp,		/* I - File to read from */
 
 
 /*
- * End of "$Id: ppds.c,v 1.23 2002/05/16 13:45:03 mike Exp $".
+ * End of "$Id: ppds.c,v 1.27 2002/12/17 19:00:17 swdev Exp $".
  */
