@@ -1,9 +1,9 @@
 /*
- * "$Id: printers.c,v 1.123 2002/06/03 19:30:11 mike Exp $"
+ * "$Id: printers.c,v 1.134 2002/12/17 19:00:18 swdev Exp $"
  *
  *   Printer routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -39,12 +39,12 @@
  *                            changed.
  *   StopPrinter()          - Stop a printer from printing any jobs...
  *   ValidateDest()         - Validate a printer/class destination.
+ *   WritePrintcap()        - Write a pseudo-printcap file for older
+ *                            applications that need it...
  *   write_irix_config()    - Update the config files used by the IRIX
  *                            desktop tools.
  *   write_irix_state()     - Update the status files used by IRIX printing
  *                            desktop tools.
- *   write_printcap()       - Write a pseudo-printcap file for older
- *                            applications that need it...
  */
 
 /*
@@ -58,7 +58,6 @@
  * Local functions...
  */
 
-static void	write_printcap(void);
 #ifdef __sgi
 static void	write_irix_config(printer_t *p);
 static void	write_irix_state(printer_t *p);
@@ -106,12 +105,6 @@ AddPrinter(const char *name)	/* I - Name of printer */
   strcpy(p->job_sheets[0], "none");
   strcpy(p->job_sheets[1], "none");
 
- /*
-  * Setup required filters and IPP attributes...
-  */
-
-  SetPrinterAttrs(p);
-
  /*
   * Insert the printer in the printer list alphabetically...
   */
@@ -137,7 +130,7 @@ AddPrinter(const char *name)	/* I - Name of printer */
   * Write a new /etc/printcap or /var/spool/lp/pstatus file.
   */
 
-  write_printcap();
+  WritePrintcap();
 
   return (p);
 }
@@ -163,7 +156,7 @@ AddPrinterFilter(printer_t  *p,		/* I - Printer to add to */
   * Range check input...
   */
 
-  if (p == NULL || filter == NULL)
+  if (p == NULL || p->filetype == NULL || filter == NULL)
     return;
 
  /*
@@ -186,9 +179,9 @@ AddPrinterFilter(printer_t  *p,		/* I - Printer to add to */
   for (temptype = MimeDatabase->types, i = MimeDatabase->num_types;
        i > 0;
        i --, temptype ++)
-    if (((super[0] == '*' && strcmp((*temptype)->super, "printer") != 0) ||
-         strcmp((*temptype)->super, super) == 0) &&
-        (type[0] == '*' || strcmp((*temptype)->type, type) == 0))
+    if (((super[0] == '*' && strcasecmp((*temptype)->super, "printer") != 0) ||
+         strcasecmp((*temptype)->super, super) == 0) &&
+        (type[0] == '*' || strcasecmp((*temptype)->type, type) == 0))
     {
       LogMessage(L_DEBUG2, "Adding filter %s/%s %s/%s %d %s",
                  (*temptype)->super, (*temptype)->type,
@@ -334,15 +327,7 @@ DeletePrinter(printer_t *p)	/* I - Printer to delete */
   {
     DefaultPrinter = Printers;
 
-#ifdef __sgi
-   /*
-    * Update the IRIX printer state for the new default printer; if
-    * no printers remain, then the default printer file will be
-    * removed...
-    */
-
-    write_irix_state(DefaultPrinter);
-#endif /* __sgi */
+    WritePrintcap();
   }
 
  /*
@@ -372,7 +357,7 @@ DeletePrinter(printer_t *p)	/* I - Printer to delete */
   * Write a new /etc/printcap file...
   */
 
-  write_printcap();
+  WritePrintcap();
 }
 
 
@@ -491,7 +476,11 @@ LoadAllPrinters(void)
 
   snprintf(line, sizeof(line), "%s/printers.conf", ServerRoot);
   if ((fp = fopen(line, "r")) == NULL)
+  {
+    LogMessage(L_ERROR, "LoadAllPrinters: Unable to open %s - %s", line,
+               strerror(errno));
     return;
+  }
 
  /*
   * Read printer configurations until we hit EOF...
@@ -559,6 +548,8 @@ LoadAllPrinters(void)
 
         line[len - 1] = '\0';
 
+        LogMessage(L_DEBUG, "LoadAllPrinters: Loading printer %s...", value);
+
         p = AddPrinter(value);
 	p->accepting = 1;
 	p->state     = IPP_PRINTER_IDLE;
@@ -844,7 +835,8 @@ SetPrinterAttrs(printer_t *p)		/* I - Printer to setup */
   ppd_option_t	*input_slot,		/* InputSlot options */
 		*media_type,		/* MediaType options */
 		*page_size,		/* PageSize options */
-		*output_bin;		/* OutputBin options */
+		*output_bin,		/* OutputBin options */
+		*media_quality;		/* EFMediaQualityMode options */
   ipp_attribute_t *attr;		/* Attribute data */
   ipp_value_t	*val;			/* Attribute value */
   int		nups[] =		/* number-up-supported values */
@@ -1026,7 +1018,7 @@ SetPrinterAttrs(printer_t *p)		/* I - Printer to setup */
                 "job-priority-supported", 100);
   ippAddInteger(p->attrs, IPP_TAG_PRINTER, IPP_TAG_INTEGER,
                 "job-priority-default", 50);
-  ippAddRange(p->attrs, IPP_TAG_PRINTER, "copies-supported", 1, 65535);
+  ippAddRange(p->attrs, IPP_TAG_PRINTER, "copies-supported", 1, MaxCopies);
   ippAddInteger(p->attrs, IPP_TAG_PRINTER, IPP_TAG_INTEGER,
                 "copies-default", 1);
   ippAddBoolean(p->attrs, IPP_TAG_PRINTER, "page-ranges-supported", 1);
@@ -1100,6 +1092,8 @@ SetPrinterAttrs(printer_t *p)		/* I - Printer to setup */
 
   printer_type = p->type;
 
+  p->raw = 0;
+
   if (p->type & CUPS_PRINTER_REMOTE)
   {
    /*
@@ -1108,6 +1102,8 @@ SetPrinterAttrs(printer_t *p)		/* I - Printer to setup */
 
     ippAddString(p->attrs, IPP_TAG_PRINTER, IPP_TAG_TEXT,
                  "printer-make-and-model", NULL, p->make_model);
+
+    p->raw = 1;
   }
   else
   {
@@ -1246,37 +1242,56 @@ SetPrinterAttrs(printer_t *p)		/* I - Printer to setup */
 	if ((page_size = ppdFindOption(ppd, "PageSize")) != NULL)
 	  num_media += page_size->num_choices;
 
-	attr = ippAddStrings(p->attrs, IPP_TAG_PRINTER, IPP_TAG_KEYWORD,
-                             "media-supported", num_media, NULL, NULL);
-        if (attr != NULL)
+	if ((media_quality = ppdFindOption(ppd, "EFMediaQualityMode")) != NULL)
+	  num_media += media_quality->num_choices;
+
+        if (num_media == 0)
 	{
-	  val = attr->values;
-
-	  if (input_slot != NULL)
-	    for (i = 0; i < input_slot->num_choices; i ++, val ++)
-	      val->string.text = strdup(input_slot->choices[i].choice);
-
-	  if (media_type != NULL)
-	    for (i = 0; i < media_type->num_choices; i ++, val ++)
-	      val->string.text = strdup(media_type->choices[i].choice);
-
-	  if (page_size != NULL)
+	  LogMessage(L_CRIT, "SetPrinterAttrs: The PPD file for printer %s "
+	                     "contains no media options and is therefore "
+			     "invalid!", p->name);
+	}
+	else
+	{
+	  attr = ippAddStrings(p->attrs, IPP_TAG_PRINTER, IPP_TAG_KEYWORD,
+                               "media-supported", num_media, NULL, NULL);
+          if (attr != NULL)
 	  {
-	    for (i = 0; i < page_size->num_choices; i ++, val ++)
-	      val->string.text = strdup(page_size->choices[i].choice);
+	    val = attr->values;
 
-	    ippAddString(p->attrs, IPP_TAG_PRINTER, IPP_TAG_KEYWORD, "media-default",
-                	 NULL, page_size->defchoice);
+	    if (input_slot != NULL)
+	      for (i = 0; i < input_slot->num_choices; i ++, val ++)
+		val->string.text = strdup(input_slot->choices[i].choice);
+
+	    if (media_type != NULL)
+	      for (i = 0; i < media_type->num_choices; i ++, val ++)
+		val->string.text = strdup(media_type->choices[i].choice);
+
+	    if (media_quality != NULL)
+	      for (i = 0; i < media_quality->num_choices; i ++, val ++)
+		val->string.text = strdup(media_quality->choices[i].choice);
+
+	    if (page_size != NULL)
+	    {
+	      for (i = 0; i < page_size->num_choices; i ++, val ++)
+		val->string.text = strdup(page_size->choices[i].choice);
+
+	      ippAddString(p->attrs, IPP_TAG_PRINTER, IPP_TAG_KEYWORD, "media-default",
+                	   NULL, page_size->defchoice);
+            }
+	    else if (input_slot != NULL)
+	      ippAddString(p->attrs, IPP_TAG_PRINTER, IPP_TAG_KEYWORD, "media-default",
+                	   NULL, input_slot->defchoice);
+	    else if (media_type != NULL)
+	      ippAddString(p->attrs, IPP_TAG_PRINTER, IPP_TAG_KEYWORD, "media-default",
+                	   NULL, media_type->defchoice);
+	    else if (media_quality != NULL)
+	      ippAddString(p->attrs, IPP_TAG_PRINTER, IPP_TAG_KEYWORD, "media-default",
+                	   NULL, media_quality->defchoice);
+	    else
+	      ippAddString(p->attrs, IPP_TAG_PRINTER, IPP_TAG_KEYWORD, "media-default",
+                	   NULL, "none");
           }
-	  else if (input_slot != NULL)
-	    ippAddString(p->attrs, IPP_TAG_PRINTER, IPP_TAG_KEYWORD, "media-default",
-                	 NULL, input_slot->defchoice);
-	  else if (media_type != NULL)
-	    ippAddString(p->attrs, IPP_TAG_PRINTER, IPP_TAG_KEYWORD, "media-default",
-                	 NULL, media_type->defchoice);
-	  else
-	    ippAddString(p->attrs, IPP_TAG_PRINTER, IPP_TAG_KEYWORD, "media-default",
-                	 NULL, "none");
         }
 
        /*
@@ -1410,7 +1425,7 @@ SetPrinterAttrs(printer_t *p)		/* I - Printer to setup */
 	  * Print all files directly...
 	  */
 
-	  AddPrinterFilter(p, "*/* 0 -");
+	  p->raw = 1;
 	}
 	else
 	{
@@ -1422,7 +1437,7 @@ SetPrinterAttrs(printer_t *p)		/* I - Printer to setup */
 	  ippAddString(p->attrs, IPP_TAG_PRINTER, IPP_TAG_TEXT,
                        "printer-make-and-model", NULL, "Local Raw Printer");
 
-	  AddPrinterFilter(p, "*/* 0 -");
+	  p->raw = 1;
 	}
       }
 
@@ -1496,12 +1511,6 @@ SetPrinterState(printer_t    *p,	/* I - Printer to change */
 
   if ((old_state == IPP_PRINTER_STOPPED) != (s == IPP_PRINTER_STOPPED))
     SaveAllPrinters();
-
- /*
-  * Check to see if any pending jobs can now be printed...
-  */
-
-  CheckJobs();
 }
 
 
@@ -1569,6 +1578,12 @@ StopPrinter(printer_t *p)	/* I - Printer to stop */
   job_t	*job;			/* Active print job */
 
 
+ /*
+  * Set the printer state...
+  */
+
+  p->state = IPP_PRINTER_STOPPED;
+
  /*
   * See if we have a job printing on this printer...
   */
@@ -1595,8 +1610,6 @@ StopPrinter(printer_t *p)	/* I - Printer to stop */
 
     SaveJob(job->id);
   }
-
-  p->state = IPP_PRINTER_STOPPED;
 }
 
 
@@ -1657,7 +1670,8 @@ ValidateDest(const char   *hostname,	/* I - Host name */
     return (NULL);
   else if (p != NULL)
   {
-    *dtype = p->type & CUPS_PRINTER_CLASS;
+    *dtype = p->type & (CUPS_PRINTER_CLASS | CUPS_PRINTER_IMPLICIT |
+                        CUPS_PRINTER_REMOTE);
     return (p->name);
   }
 
@@ -1708,7 +1722,8 @@ ValidateDest(const char   *hostname,	/* I - Host name */
     if (strcasecmp(p->hostname, localname) == 0 &&
         strcasecmp(p->name, resource) == 0)
     {
-      *dtype = p->type & (CUPS_PRINTER_CLASS | CUPS_PRINTER_REMOTE);
+      *dtype = p->type & (CUPS_PRINTER_CLASS | CUPS_PRINTER_IMPLICIT |
+                          CUPS_PRINTER_REMOTE);
       return (p->name);
     }
 
@@ -1717,17 +1732,27 @@ ValidateDest(const char   *hostname,	/* I - Host name */
 
 
 /*
- * 'write_printcap()' - Write a pseudo-printcap file for older applications
- *                      that need it...
+ * 'WritePrintcap()' - Write a pseudo-printcap file for older applications
+ *                     that need it...
  */
 
-static void
-write_printcap(void)
+void
+WritePrintcap(void)
 {
   FILE		*fp;		/* printcap file */
   printer_t	*p;		/* Current printer */
 
 
+#ifdef __sgi
+ /*
+  * Update the IRIX printer state for the default printer; if
+  * no printers remain, then the default printer file will be
+  * removed...
+  */
+
+  write_irix_state(DefaultPrinter);
+#endif /* __sgi */
+
  /*
   * See if we have a printcap file; if not, don't bother writing it.
   */
@@ -1747,7 +1772,7 @@ write_printcap(void)
   * data has come from...
   */
 
-  fputs("# This file was automatically generated by cupsd(1m) from the\n", fp);
+  fputs("# This file was automatically generated by cupsd(8) from the\n", fp);
   fprintf(fp, "# %s/printers.conf file.  All changes to this file\n",
           ServerRoot);
   fputs("# will be lost.\n", fp);
@@ -2076,5 +2101,5 @@ write_irix_state(printer_t *p)	/* I - Printer to update */
 
 
 /*
- * End of "$Id: printers.c,v 1.123 2002/06/03 19:30:11 mike Exp $".
+ * End of "$Id: printers.c,v 1.134 2002/12/17 19:00:18 swdev Exp $".
  */
