@@ -1,9 +1,9 @@
 /*
- * "$Id: server.c,v 1.3 2002/01/02 17:59:18 mike Exp $"
+ * "$Id: server.c,v 1.5 2002/12/17 19:00:18 swdev Exp $"
  *
  *   Server start/stop routines for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products, all rights reserved.
+ *   Copyright 1997-2003 by Easy Software Products, all rights reserved.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -84,18 +84,6 @@ StartServer(void)
   StartListening();
   StartBrowsing();
   StartPolling();
-
- /*
-  * If the administrator has configured the server to run as an unpriviledged
-  * user, change to that user now...
-  */
-
-  if (RunAsUser)
-  {
-    setgid(Group);
-    setgroups(0, NULL);
-    setuid(User);
-  }
 }
 
 
@@ -158,5 +146,5 @@ StopServer(void)
 
 
 /*
- * End of "$Id: server.c,v 1.3 2002/01/02 17:59:18 mike Exp $".
+ * End of "$Id: server.c,v 1.5 2002/12/17 19:00:18 swdev Exp $".
  */
