@@ -1,9 +1,9 @@
 /*
- * "$Id: lp.c,v 1.35 2002/04/22 21:09:39 mike Exp $"
+ * "$Id: lp.c,v 1.39 2002/12/17 19:00:20 swdev Exp $"
  *
  *   "lp" command for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products.
+ *   Copyright 1997-2003 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -33,12 +33,14 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <errno.h>
 #include <cups/cups.h>
 #include <cups/string.h>
 #include <cups/language.h>
 
 
 #ifndef WIN32
+#  include <unistd.h>
 #  include <signal.h>
 
 
@@ -255,12 +257,6 @@ main(int  argc,		/* I - Number of command-line arguments */
 	      num_copies = atoi(argv[i]);
 	    }
 
-	    if (num_copies < 1 || num_copies > 100)
-	    {
-	      fputs("lp: Number copies must be between 1 and 100.\n", stderr);
-	      return (1);
-	    }
-
             sprintf(buffer, "%d", num_copies);
             num_options = cupsAddOption("copies", buffer, num_options, &options);
 	    break;
@@ -295,7 +291,7 @@ main(int  argc,		/* I - Number of command-line arguments */
 	      if (i >= argc)
 	      {
 	        fprintf(stderr, "lp: Expected priority after -%c option!\n",
-		        argv[i][1]);
+		        argv[i - 1][1]);
 		return (1);
               }
 
@@ -448,6 +444,13 @@ main(int  argc,		/* I - Number of command-line arguments */
       * Print a file...
       */
 
+      if (access(argv[i], R_OK) != 0)
+      {
+        fprintf(stderr, "lp: Unable to access \"%s\" - %s\n", argv[i],
+	        strerror(errno));
+        return (1);
+      }
+
       files[num_files] = argv[i];
       num_files ++;
 
@@ -655,5 +658,5 @@ sighandler(int s)	/* I - Signal number */
 
 
 /*
- * End of "$Id: lp.c,v 1.35 2002/04/22 21:09:39 mike Exp $".
+ * End of "$Id: lp.c,v 1.39 2002/12/17 19:00:20 swdev Exp $".
  */
