@@ -1,9 +1,9 @@
 /*
- * "$Id: lpadmin.c,v 1.30 2002/05/16 13:45:04 mike Exp $"
+ * "$Id: lpadmin.c,v 1.39 2002/12/17 19:00:20 swdev Exp $"
  *
  *   "lpadmin" command for the Common UNIX Printing System (CUPS).
  *
- *   Copyright 1997-2002 by Easy Software Products.
+ *   Copyright 1997-2003 by Easy Software Products.
  *
  *   These coded instructions, statements, and computer programs are the
  *   property of Easy Software Products and are protected by Federal
@@ -36,7 +36,7 @@
  *   set_printer_model()         - Set the driver model file.
  *   set_printer_options()       - Set the printer options.
  *   validate_name()             - Make sure the printer name only contains
- *                                 letters, numbers, and the underscore...
+ *                                 valid chars...
  */
 
 /*
@@ -60,18 +60,18 @@
  * Local functions...
  */
 
-static void	add_printer_to_class(http_t *, char *, char *);
-static void	default_printer(http_t *, char *);
-static void	delete_printer(http_t *, char *);
-static void	delete_printer_from_class(http_t *, char *, char *);
-static void	enable_printer(http_t *, char *);
+static int	add_printer_to_class(http_t *, char *, char *);
+static int	default_printer(http_t *, char *);
+static int	delete_printer(http_t *, char *);
+static int	delete_printer_from_class(http_t *, char *, char *);
+static int	enable_printer(http_t *, char *);
 static char	*get_line(char *, int, FILE *fp);
-static void	set_printer_device(http_t *, char *, char *);
-static void	set_printer_file(http_t *, char *, char *);
-static void	set_printer_info(http_t *, char *, char *);
-static void	set_printer_location(http_t *, char *, char *);
-static void	set_printer_model(http_t *, char *, char *);
-static void	set_printer_options(http_t *, char *, int, cups_option_t *);
+static int	set_printer_device(http_t *, char *, char *);
+static int	set_printer_file(http_t *, char *, char *);
+static int	set_printer_info(http_t *, char *, char *);
+static int	set_printer_location(http_t *, char *, char *);
+static int	set_printer_model(http_t *, char *, char *);
+static int	set_printer_options(http_t *, char *, int, cups_option_t *);
 static int	validate_name(const char *);
 
 
@@ -90,14 +90,12 @@ main(int  argc,			/* I - Number of command-line arguments */
 		*val;		/* Pointer to allow/deny value */
   int		num_options;	/* Number of options */
   cups_option_t	*options;	/* Options */
-  http_encryption_t encryption;	/* Encryption? */
 
 
   http        = NULL;
   printer     = NULL;
   num_options = 0;
   options     = NULL;
-  encryption  = cupsEncryption();
 
   for (i = 1; i < argc; i ++)
     if (argv[i][0] == '-')
@@ -106,7 +104,8 @@ main(int  argc,			/* I - Number of command-line arguments */
         case 'c' : /* Add printer to class */
 	    if (!http)
 	    {
-              http = httpConnectEncrypt(cupsServer(), ippPort(), encryption);
+              http = httpConnectEncrypt(cupsServer(), ippPort(),
+	                                cupsEncryption());
 
 	      if (http == NULL)
 	      {
@@ -139,17 +138,19 @@ main(int  argc,			/* I - Number of command-line arguments */
 
             if (!validate_name(pclass))
 	    {
-	      fputs("lpadmin: Class name can only contain letters, numbers, and the underscore!\n", stderr);
+	      fputs("lpadmin: Class name can only contain printable characters!\n", stderr);
 	      return (1);
 	    }
 
-	    add_printer_to_class(http, printer, pclass);
+	    if (add_printer_to_class(http, printer, pclass))
+	      return (1);
 	    break;
 
         case 'd' : /* Set as default destination */
 	    if (!http)
 	    {
-              http = httpConnectEncrypt(cupsServer(), ippPort(), encryption);
+              http = httpConnectEncrypt(cupsServer(), ippPort(),
+	                                cupsEncryption());
 
 	      if (http == NULL)
 	      {
@@ -175,20 +176,25 @@ main(int  argc,			/* I - Number of command-line arguments */
 
             if (!validate_name(printer))
 	    {
-	      fputs("lpadmin: Printer name can only contain letters, numbers, and the underscore!\n", stderr);
+	      fputs("lpadmin: Printer name can only contain printable characters!\n", stderr);
 	      return (1);
 	    }
 
-            default_printer(http, printer);
+            if (default_printer(http, printer))
+	      return (1);
+
 	    i = argc;
 	    break;
 
         case 'h' : /* Connect to host */
 	    if (http)
+	    {
 	      httpClose(http);
+	      http = NULL;
+	    }
 
 	    if (argv[i][2] != '\0')
-	      http = httpConnectEncrypt(argv[i] + 2, ippPort(), encryption);
+	      cupsSetServer(argv[i] + 2);
 	    else
 	    {
 	      i ++;
@@ -198,23 +204,16 @@ main(int  argc,			/* I - Number of command-line arguments */
 	        fputs("lpadmin: Expected hostname after \'-h\' option!\n", stderr);
 		return (1);
               }
-	      else
-		http = httpConnectEncrypt(argv[i], ippPort(), encryption);
-	    }
 
-	    if (http == NULL)
-	    {
-	      perror("lpadmin: Unable to connect to server");
-	      return (1);
+              cupsSetServer(argv[i]);
 	    }
-	    else
-	      cupsSetServer(http->hostname);
 	    break;
 
         case 'i' : /* Use the specified interface script */
 	    if (!http)
 	    {
-              http = httpConnectEncrypt(cupsServer(), ippPort(), encryption);
+              http = httpConnectEncrypt(cupsServer(), ippPort(),
+	                                cupsEncryption());
 
 	      if (http == NULL)
 	      {
@@ -231,7 +230,10 @@ main(int  argc,			/* I - Number of command-line arguments */
 	    }
 
 	    if (argv[i][2])
-	      set_printer_file(http, printer, argv[i] + 2);
+	    {
+	      if (set_printer_file(http, printer, argv[i] + 2))
+	        return (1);
+            }
 	    else
 	    {
 	      i ++;
@@ -242,7 +244,8 @@ main(int  argc,			/* I - Number of command-line arguments */
 		return (1);
 	      }
 
-	      set_printer_file(http, printer, argv[i]);
+	      if (set_printer_file(http, printer, argv[i]))
+	        return (1);
 	    }
 	    break;
 
@@ -250,10 +253,10 @@ main(int  argc,			/* I - Number of command-line arguments */
 	    if (printer == NULL)
 	    {
 #ifdef HAVE_LIBSSL
-	      cupsSetEncryption(encryption = HTTP_ENCRYPT_REQUIRED);
+	      cupsSetEncryption(HTTP_ENCRYPT_REQUIRED);
 
 	      if (http)
-		httpEncryption(http, encryption);
+		httpEncryption(http, HTTP_ENCRYPT_REQUIRED);
 #else
               fprintf(stderr, "%s: Sorry, no encryption support compiled in!\n",
 	              argv[0]);
@@ -263,7 +266,8 @@ main(int  argc,			/* I - Number of command-line arguments */
 
 	    if (!http)
 	    {
-              http = httpConnectEncrypt(cupsServer(), ippPort(), encryption);
+              http = httpConnectEncrypt(cupsServer(), ippPort(),
+	                                cupsEncryption());
 
 	      if (http == NULL)
 	      {
@@ -272,13 +276,15 @@ main(int  argc,			/* I - Number of command-line arguments */
 	      }
             }
 
-            enable_printer(http, printer);
+            if (enable_printer(http, printer))
+	      return (1);
             break;
 
         case 'm' : /* Use the specified standard script/PPD file */
 	    if (!http)
 	    {
-              http = httpConnectEncrypt(cupsServer(), ippPort(), encryption);
+              http = httpConnectEncrypt(cupsServer(), ippPort(),
+	                                cupsEncryption());
 
 	      if (http == NULL)
 	      {
@@ -295,7 +301,10 @@ main(int  argc,			/* I - Number of command-line arguments */
 	    }
 
 	    if (argv[i][2])
-	      set_printer_model(http, printer, argv[i] + 2);
+	    {
+	      if (set_printer_model(http, printer, argv[i] + 2))
+	        return (1);
+	    }
 	    else
 	    {
 	      i ++;
@@ -306,7 +315,8 @@ main(int  argc,			/* I - Number of command-line arguments */
 		return (1);
 	      }
 
-	      set_printer_model(http, printer, argv[i]);
+	      if (set_printer_model(http, printer, argv[i]))
+	        return (1);
 	    }
 	    break;
 
@@ -330,7 +340,8 @@ main(int  argc,			/* I - Number of command-line arguments */
         case 'p' : /* Add/modify a printer */
 	    if (!http)
 	    {
-              http = httpConnectEncrypt(cupsServer(), ippPort(), encryption);
+              http = httpConnectEncrypt(cupsServer(), ippPort(),
+	                                cupsEncryption());
 
 	      if (http == NULL)
 	      {
@@ -356,7 +367,7 @@ main(int  argc,			/* I - Number of command-line arguments */
 
             if (!validate_name(printer))
 	    {
-	      fputs("lpadmin: Printer name can only contain letters, numbers, and the underscore!\n", stderr);
+	      fputs("lpadmin: Printer name can only contain printable characters!\n", stderr);
 	      return (1);
 	    }
 	    break;
@@ -364,7 +375,8 @@ main(int  argc,			/* I - Number of command-line arguments */
         case 'r' : /* Remove printer from class */
 	    if (!http)
 	    {
-              http = httpConnectEncrypt(cupsServer(), ippPort(), encryption);
+              http = httpConnectEncrypt(cupsServer(), ippPort(),
+	                                cupsEncryption());
 
 	      if (http == NULL)
 	      {
@@ -397,11 +409,12 @@ main(int  argc,			/* I - Number of command-line arguments */
 
             if (!validate_name(pclass))
 	    {
-	      fputs("lpadmin: Class name can only contain letters, numbers, and the underscore!\n", stderr);
+	      fputs("lpadmin: Class name can only contain printable characters!\n", stderr);
 	      return (1);
 	    }
 
-            delete_printer_from_class(http, printer, pclass);
+            if (delete_printer_from_class(http, printer, pclass))
+	      return (1);
 	    break;
 
         case 'u' : /* Allow/deny users */
@@ -437,7 +450,8 @@ main(int  argc,			/* I - Number of command-line arguments */
         case 'v' : /* Set the device-uri attribute */
 	    if (!http)
 	    {
-              http = httpConnectEncrypt(cupsServer(), ippPort(), encryption);
+              http = httpConnectEncrypt(cupsServer(), ippPort(),
+	                                cupsEncryption());
 
 	      if (http == NULL)
 	      {
@@ -454,7 +468,10 @@ main(int  argc,			/* I - Number of command-line arguments */
 	    }
 
 	    if (argv[i][2])
-	      set_printer_device(http, printer, argv[i] + 2);
+	    {
+	      if (set_printer_device(http, printer, argv[i] + 2))
+	        return (1);
+            }
 	    else
 	    {
 	      i ++;
@@ -465,14 +482,16 @@ main(int  argc,			/* I - Number of command-line arguments */
 		return (1);
 	      }
 
-	      set_printer_device(http, printer, argv[i]);
+	      if (set_printer_device(http, printer, argv[i]))
+	        return (1);
 	    }
 	    break;
 
         case 'x' : /* Delete a printer */
 	    if (!http)
 	    {
-              http = httpConnectEncrypt(cupsServer(), ippPort(), encryption);
+              http = httpConnectEncrypt(cupsServer(), ippPort(),
+	                                cupsEncryption());
 
 	      if (http == NULL)
 	      {
@@ -498,18 +517,21 @@ main(int  argc,			/* I - Number of command-line arguments */
 
             if (!validate_name(printer))
 	    {
-	      fputs("lpadmin: Printer name can only contain letters, numbers, and the underscore!\n", stderr);
+	      fputs("lpadmin: Printer name can only contain printable characters!\n", stderr);
 	      return (1);
 	    }
 
-            delete_printer(http, printer);
+            if (delete_printer(http, printer))
+	      return (1);
+
 	    i = argc;
 	    break;
 
         case 'D' : /* Set the printer-info attribute */
 	    if (!http)
 	    {
-              http = httpConnectEncrypt(cupsServer(), ippPort(), encryption);
+              http = httpConnectEncrypt(cupsServer(), ippPort(),
+	                                cupsEncryption());
 
 	      if (http == NULL)
 	      {
@@ -526,7 +548,10 @@ main(int  argc,			/* I - Number of command-line arguments */
 	    }
 
 	    if (argv[i][2])
-	      set_printer_info(http, printer, argv[i] + 2);
+	    {
+	      if (set_printer_info(http, printer, argv[i] + 2))
+	        return (1);
+	    }
 	    else
 	    {
 	      i ++;
@@ -537,7 +562,8 @@ main(int  argc,			/* I - Number of command-line arguments */
 		return (1);
 	      }
 
-	      set_printer_info(http, printer, argv[i]);
+	      if (set_printer_info(http, printer, argv[i]))
+	        return (1);
 	    }
 	    break;
 
@@ -556,7 +582,8 @@ main(int  argc,			/* I - Number of command-line arguments */
         case 'L' : /* Set the printer-location attribute */
 	    if (!http)
 	    {
-              http = httpConnectEncrypt(cupsServer(), ippPort(), encryption);
+              http = httpConnectEncrypt(cupsServer(), ippPort(),
+	                                cupsEncryption());
 
 	      if (http == NULL)
 	      {
@@ -573,7 +600,10 @@ main(int  argc,			/* I - Number of command-line arguments */
 	    }
 
 	    if (argv[i][2])
-	      set_printer_location(http, printer, argv[i] + 2);
+	    {
+	      if (set_printer_location(http, printer, argv[i] + 2))
+	        return (1);
+            }
 	    else
 	    {
 	      i ++;
@@ -584,14 +614,16 @@ main(int  argc,			/* I - Number of command-line arguments */
 		return (1);
 	      }
 
-	      set_printer_location(http, printer, argv[i]);
+	      if (set_printer_location(http, printer, argv[i]))
+	        return (1);
 	    }
 	    break;
 
         case 'P' : /* Use the specified PPD file */
 	    if (!http)
 	    {
-              http = httpConnectEncrypt(cupsServer(), ippPort(), encryption);
+              http = httpConnectEncrypt(cupsServer(), ippPort(),
+	                                cupsEncryption());
 
 	      if (http == NULL)
 	      {
@@ -608,7 +640,10 @@ main(int  argc,			/* I - Number of command-line arguments */
 	    }
 
 	    if (argv[i][2])
-	      set_printer_file(http, printer, argv[i] + 2);
+	    {
+	      if (set_printer_file(http, printer, argv[i] + 2))
+	        return (1);
+	    }
 	    else
 	    {
 	      i ++;
@@ -619,7 +654,8 @@ main(int  argc,			/* I - Number of command-line arguments */
 		return (1);
 	      }
 
-	      set_printer_file(http, printer, argv[i]);
+	      if (set_printer_file(http, printer, argv[i]))
+	        return (1);
 	    }
 	    break;
 
@@ -641,7 +677,7 @@ main(int  argc,			/* I - Number of command-line arguments */
   {
     if (!http)
     {
-      http = httpConnectEncrypt(cupsServer(), ippPort(), encryption);
+      http = httpConnectEncrypt(cupsServer(), ippPort(), cupsEncryption());
 
       if (http == NULL)
       {
@@ -657,7 +693,8 @@ main(int  argc,			/* I - Number of command-line arguments */
       return (1);
     }
 
-    set_printer_options(http, printer, num_options, options);
+    if (set_printer_options(http, printer, num_options, options))
+      return (1);
   }
 
   if (printer == NULL)
@@ -684,7 +721,7 @@ main(int  argc,			/* I - Number of command-line arguments */
  * 'add_printer_to_class()' - Add a printer to a class.
  */
 
-static void
+static int				/* O - 0 on success, 1 on fail */
 add_printer_to_class(http_t *http,	/* I - Server connection */
                      char   *printer,	/* I - Printer to add */
 		     char   *pclass)	/* I - Class to add to */
@@ -773,7 +810,7 @@ add_printer_to_class(http_t *http,	/* I - Server connection */
 	        printer, pclass);
         ippDelete(request);
 	ippDelete(response);
-	return;
+	return (0);
       }
 
  /*
@@ -806,15 +843,26 @@ add_printer_to_class(http_t *http,	/* I - Server connection */
   ippDelete(response);
 
   if ((response = cupsDoRequest(http, request, "/admin/")) == NULL)
+  {
     fprintf(stderr, "lpadmin: add-class failed: %s\n",
             ippErrorString(cupsLastError()));
-  else
+
+    return (1);
+  }
+  else if (response->request.status.status_code > IPP_OK_CONFLICT)
   {
-    if (response->request.status.status_code > IPP_OK_CONFLICT)
-      fprintf(stderr, "lpadmin: add-class failed: %s\n",
-              ippErrorString(response->request.status.status_code));
+    fprintf(stderr, "lpadmin: add-class failed: %s\n",
+            ippErrorString(response->request.status.status_code));
 
     ippDelete(response);
+
+    return (1);
+  }
+  else
+  {
+    ippDelete(response);
+
+    return (0);
   }
 }
 
@@ -823,7 +871,7 @@ add_printer_to_class(http_t *http,	/* I - Server connection */
  * 'default_printer()' - Set the default printing destination.
  */
 
-static void
+static int				/* O - 0 on success, 1 on fail */
 default_printer(http_t *http,		/* I - Server connection */
                 char   *printer)	/* I - Printer name */
 {
@@ -867,15 +915,26 @@ default_printer(http_t *http,		/* I - Server connection */
   */
 
   if ((response = cupsDoRequest(http, request, "/admin/")) == NULL)
+  {
     fprintf(stderr, "lpadmin: set-default failed: %s\n",
             ippErrorString(cupsLastError()));
-  else
+
+    return (1);
+  }
+  else if (response->request.status.status_code > IPP_OK_CONFLICT)
   {
-    if (response->request.status.status_code > IPP_OK_CONFLICT)
-      fprintf(stderr, "lpadmin: set-default failed: %s\n",
-              ippErrorString(response->request.status.status_code));
+    fprintf(stderr, "lpadmin: set-default failed: %s\n",
+            ippErrorString(response->request.status.status_code));
 
     ippDelete(response);
+
+    return (1);
+  }
+  else
+  {
+    ippDelete(response);
+
+    return (0);
   }
 }
 
@@ -884,7 +943,7 @@ default_printer(http_t *http,		/* I - Server connection */
  * 'delete_printer()' - Delete a printer from the system...
  */
 
-static void
+static int				/* O - 0 on success, 1 on fail */
 delete_printer(http_t *http,		/* I - Server connection */
                char   *printer)		/* I - Printer to delete */
 {
@@ -928,15 +987,26 @@ delete_printer(http_t *http,		/* I - Server connection */
   */
 
   if ((response = cupsDoRequest(http, request, "/admin/")) == NULL)
+  {
     fprintf(stderr, "lpadmin: delete-printer failed: %s\n",
             ippErrorString(cupsLastError()));
-  else
+
+    return (1);
+  }
+  else if (response->request.status.status_code > IPP_OK_CONFLICT)
   {
-    if (response->request.status.status_code > IPP_OK_CONFLICT)
-      fprintf(stderr, "lpadmin: delete-printer failed: %s\n",
-              ippErrorString(response->request.status.status_code));
+    fprintf(stderr, "lpadmin: delete-printer failed: %s\n",
+            ippErrorString(response->request.status.status_code));
 
     ippDelete(response);
+
+    return (1);
+  }
+  else
+  {
+    ippDelete(response);
+
+    return (0);
   }
 }
 
@@ -945,10 +1015,12 @@ delete_printer(http_t *http,		/* I - Server connection */
  * 'delete_printer_from_class()' - Delete a printer from a class.
  */
 
-static void
-delete_printer_from_class(http_t *http,		/* I - Server connection */
-                          char   *printer,	/* I - Printer to remove */
-			  char   *pclass)	/* I - Class to remove from */
+static int				/* O - 0 on success, 1 on fail */
+delete_printer_from_class(http_t *http,	/* I - Server connection */
+                          char   *printer,
+			  		/* I - Printer to remove */
+			  char   *pclass)
+			  		/* I - Class to remove from */
 {
   int		i, j, k;		/* Looping vars */
   ipp_t		*request,		/* IPP Request */
@@ -998,7 +1070,7 @@ delete_printer_from_class(http_t *http,		/* I - Server connection */
   {
     ippDelete(response);
     fprintf(stderr, "lpadmin: Class %s does not exist!\n", pclass);
-    return;
+    return (1);
   }
 
  /*
@@ -1009,7 +1081,7 @@ delete_printer_from_class(http_t *http,		/* I - Server connection */
   {
     ippDelete(response);
     fputs("lpadmin: No member names were seen!\n", stderr);
-    return;
+    return (1);
   }
 
   for (i = 0; i < members->num_values; i ++)
@@ -1021,7 +1093,7 @@ delete_printer_from_class(http_t *http,		/* I - Server connection */
     fprintf(stderr, "lpadmin: Printer %s is not a member of class %s.\n",
 	    printer, pclass);
     ippDelete(response);
-    return;
+    return (1);
   }
 
   if (members->num_values == 1)
@@ -1099,15 +1171,25 @@ delete_printer_from_class(http_t *http,		/* I - Server connection */
   ippDelete(response);
 
   if ((response = cupsDoRequest(http, request, "/admin/")) == NULL)
+  {
     fprintf(stderr, "lpadmin: add/delete-class failed: %s\n",
             ippErrorString(cupsLastError()));
-  else
+    return (1);
+  }
+  else if (response->request.status.status_code > IPP_OK_CONFLICT)
   {
-    if (response->request.status.status_code > IPP_OK_CONFLICT)
-      fprintf(stderr, "lpadmin: add/delete-class failed: %s\n",
-              ippErrorString(response->request.status.status_code));
+    fprintf(stderr, "lpadmin: add/delete-class failed: %s\n",
+            ippErrorString(response->request.status.status_code));
 
     ippDelete(response);
+
+    return (1);
+  }
+  else
+  {
+    ippDelete(response);
+
+    return (0);
   }
 }
 
@@ -1116,7 +1198,7 @@ delete_printer_from_class(http_t *http,		/* I - Server connection */
  * 'enable_printer()' - Enable a printer...
  */
 
-static void
+static int				/* O - 0 on success, 1 on fail */
 enable_printer(http_t *http,		/* I - Server connection */
                char   *printer)		/* I - Printer to enable */
 {
@@ -1167,15 +1249,25 @@ enable_printer(http_t *http,		/* I - Server connection */
   */
 
   if ((response = cupsDoRequest(http, request, "/admin/")) == NULL)
-    fprintf(stderr, "lpadmin: add-printer failed: %s\n",
-            ippErrorString(cupsLastError()));
-  else
   {
-    if (response->request.status.status_code > IPP_OK_CONFLICT)
-      fprintf(stderr, "lpadmin: add-printer failed: %s\n",
-              ippErrorString(response->request.status.status_code));
+    fprintf(stderr, "lpadmin: add-printer (enable) failed: %s\n",
+            ippErrorString(cupsLastError()));
+    return (1);
+  }
+  else if (response->request.status.status_code > IPP_OK_CONFLICT)
+  {
+    fprintf(stderr, "lpadmin: add-printer (enable) failed: %s\n",
+            ippErrorString(response->request.status.status_code));
 
     ippDelete(response);
+
+    return (1);
+  }
+  else
+  {
+    ippDelete(response);
+
+    return (0);
   }
 }
 
@@ -1184,13 +1276,13 @@ enable_printer(http_t *http,		/* I - Server connection */
  * 'get_line()' - Get a line that is terminated by a LF, CR, or CR LF.
  */
 
-static char *		/* O - Pointer to buf or NULL on EOF */
-get_line(char *buf,	/* I - Line buffer */
-         int  length,	/* I - Length of buffer */
-	 FILE *fp)	/* I - File to read from */
+static char *				/* O - Pointer to buf or NULL on EOF */
+get_line(char *buf,			/* I - Line buffer */
+         int  length,			/* I - Length of buffer */
+	 FILE *fp)			/* I - File to read from */
 {
-  char	*bufptr;	/* Pointer into buffer */
-  int	ch;		/* Character from file */
+  char	*bufptr;			/* Pointer into buffer */
+  int	ch;				/* Character from file */
 
 
   length --;
@@ -1232,7 +1324,7 @@ get_line(char *buf,	/* I - Line buffer */
  * 'set_printer_device()' - Set the device-uri attribute.
  */
 
-static void
+static int				/* O - 0 on success, 1 on fail */
 set_printer_device(http_t *http,	/* I - Server connection */
                    char   *printer,	/* I - Printer */
 		   char   *device)	/* I - New device URI */
@@ -1296,15 +1388,25 @@ set_printer_device(http_t *http,	/* I - Server connection */
   */
 
   if ((response = cupsDoRequest(http, request, "/admin/")) == NULL)
-    fprintf(stderr, "lpadmin: add-printer failed: %s\n",
-            ippErrorString(cupsLastError()));
-  else
   {
-    if (response->request.status.status_code > IPP_OK_CONFLICT)
-      fprintf(stderr, "lpadmin: add-printer failed: %s\n",
-              ippErrorString(response->request.status.status_code));
+    fprintf(stderr, "lpadmin: add-printer (set device) failed: %s\n",
+            ippErrorString(cupsLastError()));
+    return (1);
+  }
+  else if (response->request.status.status_code > IPP_OK_CONFLICT)
+  {
+    fprintf(stderr, "lpadmin: add-printer (set device) failed: %s\n",
+            ippErrorString(response->request.status.status_code));
 
     ippDelete(response);
+
+    return (1);
+  }
+  else
+  {
+    ippDelete(response);
+
+    return (0);
   }
 }
 
@@ -1313,11 +1415,12 @@ set_printer_device(http_t *http,	/* I - Server connection */
  * 'set_printer_file()' - Set the interface script or PPD file.
  */
 
-static void
+static int				/* O - 0 on success, 1 on fail */
 set_printer_file(http_t *http,		/* I - Server connection */
                  char   *printer,	/* I - Printer */
 		 char   *file)		/* I - PPD file or interface script */
 {
+  ipp_status_t	status;			/* IPP status code */
   ipp_t		*request,		/* IPP Request */
 		*response;		/* IPP Response */
   cups_lang_t	*language;		/* Default language */
@@ -1347,7 +1450,7 @@ set_printer_file(http_t *http,		/* I - Server connection */
     if ((fd = cupsTempFd(tempfile, sizeof(tempfile))) < 0)
     {
       perror("lpadmin: Unable to create temporary file");
-      return;
+      return (1);
     }
 
     if ((gz = gzopen(file, "rb")) == NULL)
@@ -1355,7 +1458,7 @@ set_printer_file(http_t *http,		/* I - Server connection */
       perror("lpadmin: Unable to open file");
       close(fd);
       unlink(tempfile);
-      return;
+      return (1);
     }
 
     while ((bytes = gzread(gz, buffer, sizeof(buffer))) > 0)
@@ -1400,14 +1503,10 @@ set_printer_file(http_t *http,		/* I - Server connection */
   */
 
   if ((response = cupsDoFileRequest(http, request, "/admin/", file)) == NULL)
-    fprintf(stderr, "lpadmin: add-printer failed: %s\n",
-            ippErrorString(cupsLastError()));
+    status = cupsLastError();
   else
   {
-    if (response->request.status.status_code > IPP_OK_CONFLICT)
-      fprintf(stderr, "lpadmin: add-printer failed: %s\n",
-              ippErrorString(response->request.status.status_code));
-
+    status = response->request.status.status_code;
     ippDelete(response);
   }
 
@@ -1419,6 +1518,16 @@ set_printer_file(http_t *http,		/* I - Server connection */
   if (file == tempfile)
     unlink(tempfile);
 #endif /* HAVE_LIBZ */
+
+  if (status > IPP_OK_CONFLICT)
+  {
+    fprintf(stderr, "lpadmin: add-printer (set model) failed: %s\n",
+            ippErrorString(status));
+
+    return (1);
+  }
+  else
+    return (0);
 }
 
 
@@ -1426,7 +1535,7 @@ set_printer_file(http_t *http,		/* I - Server connection */
  * 'set_printer_info()' - Set the printer description string.
  */
 
-static void
+static int				/* O - 0 on success, 1 on fail */
 set_printer_info(http_t *http,		/* I - Server connection */
                  char   *printer,	/* I - Printer */
 		 char   *info)		/* I - New description string */
@@ -1479,15 +1588,25 @@ set_printer_info(http_t *http,		/* I - Server connection */
   */
 
   if ((response = cupsDoRequest(http, request, "/admin/")) == NULL)
-    fprintf(stderr, "lpadmin: add-printer failed: %s\n",
-            ippErrorString(cupsLastError()));
-  else
   {
-    if (response->request.status.status_code > IPP_OK_CONFLICT)
-      fprintf(stderr, "lpadmin: add-printer failed: %s\n",
-              ippErrorString(response->request.status.status_code));
+    fprintf(stderr, "lpadmin: add-printer (set description) failed: %s\n",
+            ippErrorString(cupsLastError()));
+    return (1);
+  }
+  else if (response->request.status.status_code > IPP_OK_CONFLICT)
+  {
+    fprintf(stderr, "lpadmin: add-printer (set description) failed: %s\n",
+            ippErrorString(response->request.status.status_code));
 
     ippDelete(response);
+
+    return (1);
+  }
+  else
+  {
+    ippDelete(response);
+
+    return (0);
   }
 }
 
@@ -1496,7 +1615,7 @@ set_printer_info(http_t *http,		/* I - Server connection */
  * 'set_printer_location()' - Set the printer location string.
  */
 
-static void
+static int				/* O - 0 on success, 1 on fail */
 set_printer_location(http_t *http,	/* I - Server connection */
                      char   *printer,	/* I - Printer */
 		     char   *location)	/* I - New location string */
@@ -1549,15 +1668,26 @@ set_printer_location(http_t *http,	/* I - Server connection */
   */
 
   if ((response = cupsDoRequest(http, request, "/admin/")) == NULL)
-    fprintf(stderr, "lpadmin: add-printer failed: %s\n",
-            ippErrorString(cupsLastError()));
-  else
   {
-    if (response->request.status.status_code > IPP_OK_CONFLICT)
-      fprintf(stderr, "lpadmin: add-printer failed: %s\n",
-              ippErrorString(response->request.status.status_code));
+    fprintf(stderr, "lpadmin: add-printer (set location) failed: %s\n",
+            ippErrorString(cupsLastError()));
+
+    return (1);
+  }
+  else if (response->request.status.status_code > IPP_OK_CONFLICT)
+  {
+    fprintf(stderr, "lpadmin: add-printer (set location) failed: %s\n",
+            ippErrorString(response->request.status.status_code));
 
     ippDelete(response);
+
+    return (1);
+  }
+  else
+  {
+    ippDelete(response);
+
+    return (0);
   }
 }
 
@@ -1566,7 +1696,7 @@ set_printer_location(http_t *http,	/* I - Server connection */
  * 'set_printer_model()' - Set the driver model file.
  */
 
-static void
+static int				/* O - 0 on success, 1 on fail */
 set_printer_model(http_t *http,		/* I - Server connection */
                   char   *printer,	/* I - Printer */
 		  char   *model)	/* I - Driver model file */
@@ -1613,15 +1743,26 @@ set_printer_model(http_t *http,		/* I - Server connection */
   */
 
   if ((response = cupsDoRequest(http, request, "/admin/")) == NULL)
-    fprintf(stderr, "lpadmin: add-printer failed: %s\n",
-            ippErrorString(cupsLastError()));
-  else
   {
-    if (response->request.status.status_code > IPP_OK_CONFLICT)
-      fprintf(stderr, "lpadmin: add-printer failed: %s\n",
-              ippErrorString(response->request.status.status_code));
+    fprintf(stderr, "lpadmin: add-printer (set model) failed: %s\n",
+            ippErrorString(cupsLastError()));
+
+    return (1);
+  }
+  else if (response->request.status.status_code > IPP_OK_CONFLICT)
+  {
+    fprintf(stderr, "lpadmin: add-printer (set model) failed: %s\n",
+            ippErrorString(response->request.status.status_code));
 
     ippDelete(response);
+
+    return (1);
+  }
+  else
+  {
+    ippDelete(response);
+
+    return (0);
   }
 }
 
@@ -1630,11 +1771,14 @@ set_printer_model(http_t *http,		/* I - Server connection */
  * 'set_printer_options()' - Set the printer options.
  */
 
-static void
-set_printer_options(http_t        *http,	/* I - Server connection */
-                    char          *printer,	/* I - Printer */
-		    int           num_options,	/* I - Number of options */
-		    cups_option_t *options)	/* I - Options */
+static int				/* O - 0 on success, 1 on fail */
+set_printer_options(http_t        *http,/* I - Server connection */
+                    char          *printer,
+		    			/* I - Printer */
+		    int           num_options,
+		    			/* I - Number of options */
+		    cups_option_t *options)
+		    			/* I - Options */
 {
   ipp_t		*request,		/* IPP Request */
 		*response;		/* IPP Response */
@@ -1702,7 +1846,10 @@ set_printer_options(http_t        *http,	/* I - Server connection */
     if ((attr = ippFindAttribute(response, "printer-type", IPP_TAG_ENUM)) != NULL)
     {
       if (attr->values[0].integer & (CUPS_PRINTER_CLASS | CUPS_PRINTER_IMPLICIT))
+      {
         op = CUPS_ADD_CLASS;
+        snprintf(uri, sizeof(uri), "ipp://localhost/classes/%s", printer);
+      }
     }
 
     ippDelete(response);
@@ -1816,52 +1963,60 @@ set_printer_options(http_t        *http,	/* I - Server connection */
   */
 
   if (response == NULL)
+  {
     fprintf(stderr, "lpadmin: %s failed: %s\n",
             op == CUPS_ADD_PRINTER ? "add-printer" : "add-class",
             ippErrorString(cupsLastError()));
-  else
+
+    return (1);
+  }
+  else if (response->request.status.status_code > IPP_OK_CONFLICT)
   {
-    if (response->request.status.status_code > IPP_OK_CONFLICT)
-      fprintf(stderr, "lpadmin: %s failed: %s\n",
-              op == CUPS_ADD_PRINTER ? "add-printer" : "add-class",
-              ippErrorString(response->request.status.status_code));
+    fprintf(stderr, "lpadmin: %s failed: %s\n",
+            op == CUPS_ADD_PRINTER ? "add-printer" : "add-class",
+            ippErrorString(response->request.status.status_code));
 
     ippDelete(response);
+
+    return (1);
+  }
+  else
+  {
+    ippDelete(response);
+
+    return (0);
   }
 }
 
 
 /*
- * 'validate_name()' - Make sure the printer name only contains letters,
- *                     numbers, and the underscore...
+ * 'validate_name()' - Make sure the printer name only contains valid chars.
  */
 
 static int			/* O - 0 if name is no good, 1 if name is good */
 validate_name(const char *name)	/* I - Name to check */
 {
- /*
-  * Don't allow names to start with a digit...
-  */
+  const char	*ptr;		/* Pointer into name */
 
-  if (isdigit(*name))
-    return (0);
 
  /*
   * Scan the whole name...
   */
 
-  while (*name)
-    if (*name == '@')
-      return (1);
-    else if (!isalnum(*name) && *name != '_')
+  for (ptr = name; *ptr; ptr ++)
+    if (*ptr == '@')
+      break;
+    else if ((*ptr >= 0 && *ptr <= ' ') || *ptr == 127 || *ptr == '/')
       return (0);
-    else
-      name ++;
 
-  return (1);
+ /*
+  * All the characters are good; validate the length, too...
+  */
+
+  return ((ptr - name) < 128);
 }
 
 
 /*
- * End of "$Id: lpadmin.c,v 1.30 2002/05/16 13:45:04 mike Exp $".
+ * End of "$Id: lpadmin.c,v 1.39 2002/12/17 19:00:20 swdev Exp $".
  */
