@@ -1,11 +1,11 @@
 #!/bin/sh
 #
-# "$Id: run-stp-tests.sh,v 1.10 2002/05/14 01:14:31 mike Exp $"
+# "$Id: run-stp-tests.sh,v 1.14 2002/12/17 19:00:25 swdev Exp $"
 #
 #   Perform the complete set of IPP compliance tests specified in the
 #   CUPS Software Test Plan.
 #
-#   Copyright 1997-2002 by Easy Software Products, all rights reserved.
+#   Copyright 1997-2003 by Easy Software Products, all rights reserved.
 #
 #   These coded instructions, statements, and computer programs are the
 #   property of Easy Software Products and are protected by Federal
@@ -30,6 +30,52 @@
 
 make
 
+#
+# Greet the tester...
+#
+
+echo "Welcome to the CUPS Automated Test Script."
+echo ""
+echo "Before we begin, it is important that you understand that the larger"
+echo "tests require significant amounts of RAM and disk space.  If you"
+echo "attempt to run one of the big tests on a system that lacks sufficient"
+echo "disk and virtual memory, the UNIX kernel might decide to kill one or"
+echo "more system processes that you've grown attached to, like the X"
+echo "server.  The question you may want to ask yourself before running a"
+echo "large test is: Do you feel lucky?"
+echo ""
+echo "OK, now that we have the Dirty Harry quote out of the way, please"
+echo "choose the type of test you wish to perform:"
+echo ""
+echo "1 - Basic conformance test with no load testing (all systems)"
+echo "2 - Basic conformance test with some load testing (minimum 256MB VM)"
+echo "3 - Basic conformance test with extreme load testing (minimum 1024MB VM)"
+echo ""
+echo "Please enter the number of the test you wish to perform:"
+
+read testtype
+
+case "$testtype" in
+	2)
+		echo "Running the medium tests (2)"
+		nprinters1=10
+		nprinters2=20
+		pjobs=20
+		;;
+	3)
+		echo "Running the extreme tests (3)"
+		nprinters1=500
+		nprinters2=1000
+		pjobs=100
+		;;
+	*)
+		echo "Running the timid tests (1)"
+		nprinters1=0
+		nprinters2=0
+		pjobs=0
+		;;
+esac
+
 #
 # Information for the server/tests...
 #
@@ -126,6 +172,47 @@ EOF
 touch /tmp/$user/classes.conf
 touch /tmp/$user/printers.conf
 
+#
+# Setup lots of test queues - 500 with PPD files, 500 without...
+#
+
+i=1
+while test $i -le $nprinters1; do
+	cat >>/tmp/$user/printers.conf <<EOF
+<Printer test-$i>
+Accepting Yes
+DeviceURI file:/dev/null
+Info Test PS printer $i
+JobSheets none none
+Location CUPS test suite
+State Idle
+StateMessage Printer $1 is idle.
+</Printer>
+EOF
+
+	cp testps.ppd /tmp/$user/ppd/test-$i.ppd
+
+	i=`expr $i + 1`
+done
+
+while test $i -le $nprinters2; do
+	cat >>/tmp/$user/printers.conf <<EOF
+<Printer test-$i>
+Accepting Yes
+DeviceURI file:/dev/null
+Info Test raw printer $i
+JobSheets none none
+Location CUPS test suite
+State Idle
+StateMessage Printer $1 is idle.
+</Printer>
+EOF
+
+	i=`expr $i + 1`
+done
+
+cp /tmp/$user/printers.conf /tmp/$user/printers.conf.orig
+
 cp $root/conf/mime.types /tmp/$user/mime.types
 cp $root/conf/mime.convs /tmp/$user/mime.convs
 
@@ -152,6 +239,14 @@ fi
 
 export DYLD_LIBRARY_PATH
 
+if test "x$SHLIB_PATH" = x; then
+	SHLIB_PATH="$root/cups:$root/filter"
+else
+	SHLIB_PATH="$root/cups:$root/filter:$SHLIB_PATH"
+fi
+
+export SHLIB_PATH
+
 CUPS_SERVERROOT=/tmp/$user; export CUPS_SERVERROOT
 CUPS_DATADIR=/tmp/$user/share; export CUPS_DATADIR
 
@@ -242,7 +337,7 @@ for file in 5*.sh; do
 	echo "" >>$strfile
 	echo "\"$file\":" >>$strfile
 
-	sh $file >>$strfile
+	sh $file $pjobs >>$strfile
 	status=$?
 
 	if test $status != 0; then
@@ -326,5 +421,5 @@ echo "    $pdffile"
 echo ""
 
 #
-# End of "$Id: run-stp-tests.sh,v 1.10 2002/05/14 01:14:31 mike Exp $"
+# End of "$Id: run-stp-tests.sh,v 1.14 2002/12/17 19:00:25 swdev Exp $"
 #
